/*  ===========================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005
 *
 *   Use of this software is controlled by the terms and conditions found in
 *   the license agreement under which this software has been supplied.
 *   ==========================================================================
 */

/** ===========================================================================
 *
 *   @file  Cache_example.c
 *
 *   @path  $(CSLPATH)\example\cache\src
 *
 *   @desc  Example of CACHE
 *
 * ============================================================================
 *   @n <b> Example Description </b>
 *   @n Usage of cache CSL API for the following operations
 *       - Invalidate all L2 and L1D cache
 *       - Write back invalidate all L2 and L1D cache
 *       - Write back all L2 and L1D cache
 *       - Invalidate the L2 and L1D cache
 *       - Write back invalidate L2 and L1D cache
 *       - Write back L2 and L1D cache
 *       - L2 normal mode
 *       - Freeze operation on L2 cache
 *
 * ===========================================================================
 *
 *   <b> Test Procedure </b>
 *   @verbatim
 *      1. Configure the CCS setup to work with the emulator being used
 *      2. Please refer CCS manual for setup configuration and loading
 *         proper GEL file
 *      3. Launch CCS window
 *      4. Open project Cache_example.pjt
 *      5. Build the project and load the .out file of the project.
 *      6. The example can be executed from the main().
 *
 *   @endverbatim
 *
 *
 * =============================================================================
 */

/* ============================================================================
 *  Revision History
 *  ===============
 *  26-Jul-2005 sd     File Created.
 *
 * ============================================================================
 */
#include <stdio.h>
#include <csl_cache.h>
#include <_csl_cache.h>

/* EMIF Memory area */
#define EMIF_MEM        0x80000000

/* Globals variables used for storing the cache data */
Uint32      temp;
Uint32      Data_1;
Uint32      Data_2;
Uint32      Data_3;
Uint32      Data_4;

/* Foraward declarations */
void subExampleEndPrint(void);
void exampleEndPrint (void);


/*
 * =============================================================================
 *   @func   main
 *
 *   @desc
 *     This is the main routine for the file.
 *
 * =============================================================================
 */

void main(void)
{
    Uint32          cacheExampleFail = 0;
    Uint16          loopCount;
    Uint16          count;
    CACHE_L2Size    localSize[4];


    /* make sure L1P is turned off */
    CACHE_setL1pSize(CACHE_L1_0KCACHE);

    /* make sure that L1D is turned off */
    CACHE_setL1dSize(CACHE_L1_0KCACHE);

    /* make sure that L2 is turned off */
    CACHE_setL2Size(CACHE_0KCACHE);

    /* Array used for setting L2 cache size */
    localSize[0] = CACHE_32KCACHE;
    localSize[1] = CACHE_64KCACHE;
    localSize[2] = CACHE_128KCACHE;
    localSize[3] = CACHE_256KCACHE;

    /* This loop is used to display the L2 cache size while performing different
     * operations on different L2 cache sizes
     */
    for(loopCount = 0; loopCount <4; loopCount++) {
        switch(loopCount) {
            case 0:
                printf("\n");
                subExampleEndPrint();
                printf("CACHE: Performing operations on 32K L2 cache.\n");
                subExampleEndPrint();
                break;

            case 1:
                printf("\n");
                subExampleEndPrint();
                printf("CACHE: Performing operations on 64K L2 cache.\n");
                subExampleEndPrint();
                break;
            case 2:
                printf("\n");
                subExampleEndPrint();
                printf("CACHE: Performing operations on 128K L2 cache.\n");
                subExampleEndPrint();
                break;

            case 3:
                printf("\n");
                subExampleEndPrint();
                printf("CACHE: Performing operations on 256K L2 cache.\n");
                subExampleEndPrint();
                break;

            default:
                break;
        }

        for(count = 1; count <= 7; count++) {
            /* set a test memory location */
            *(Uint32*)EMIF_MEM = 0xABCDABCD;

            Data_2 = 0xa5a5a5a5;

            /* set Mar bit for EMIF_MEM */
            CACHE_enableCaching(CACHE_EMIFA_CE00);

            /* turn on the L2 cache. */
            CACHE_setL2Size( localSize[loopCount]);

            /* read (allocate into cache) */
            Data_1 = *(Uint32*)EMIF_MEM;

            /* write (dirty the cache line) */
            *(Uint32*)EMIF_MEM = Data_2;

            /* Read the data back (make sure it matches Data_2 */
            Data_3 = *(Uint32*)EMIF_MEM;

            if(Data_2 != Data_3)
                printf("CACHE: Data read from cache is not same as \
                        data written.\n");

            switch(count) {
                /* test #1 - invAllL2 */
                case 1:
                    /* Invalidate all the L2 cache */
                    CACHE_invAllL2(CACHE_WAIT);

                    for( temp =0 ; temp < 1000; temp++)
                    {}

                    /* read, this data should match data_1 */
                    Data_4 = *(Uint32*)EMIF_MEM;
                    if(Data_1 != Data_4) {
                        printf("CACHE: Invalidate the All L2 cache Failed.\n");
                        cacheExampleFail++;
                    }
                    else {
                        printf("CACHE: Invalidate the All L2 cache Passed.\n");
                    }

                    break;

                /* Test #2 - wbInvAllL2 */
                case 2:
                    /* Writeback invalidate all the L2 cache */
                    CACHE_wbInvAllL2(CACHE_WAIT);

                    for( temp =0 ; temp < 1000; temp++)
                    {}

                    /* read, this data should match data_2 */
                    Data_4 = *(Uint32*)EMIF_MEM;
                    if(Data_2 != Data_4) {
                        printf("CACHE: Write back invalidate All");
                        printf("L2 cache Failed.\n");
                        cacheExampleFail++;
                    }
                    else {
                        printf("CACHE: Write back invalidate All");
                        printf("L2 cache Passed.\n");
                    }
                    break;

                /* Test #3 wbAll */
                case 3:
                    /* Writeback all the L2 cache */
                    CACHE_wbAllL2(CACHE_WAIT);

                    for( temp =0 ; temp < 1000; temp++)
                    {}

                    /* read, this data should match data_2 */
                    Data_4 = *(Uint32*)EMIF_MEM;
                    if(Data_2 != Data_4){
                        printf("CACHE: Write back All L2 cache Failed.\n");
                        cacheExampleFail++;
                    }
                    else {
                        printf("CACHE: Write back All L2 cache Passed.\n");
                    }
                    break;

                /* test #4 - invL2 */
                case 4:
                    /* Invalidate the L2 cache */
                    CACHE_invL2((Uint32*)EMIF_MEM, 4, CACHE_WAIT);

                    /* read, this data should match data_1 */
                    Data_4 = *(Uint32*)EMIF_MEM;
                    if(Data_1 != Data_4) {
                        printf("CACHE: Invalidate the L2 cache Failed.\n");
                        cacheExampleFail++;
                    }
                    else {
                        printf("CACHE: Invalidate the L2 cache Passed.\n");
                    }
                    break;

                /* Test #5 - wbInv */
                case 5:
                    /* Writeback invalidate the L2 cache */
                    CACHE_wbInvL2((Uint32*)EMIF_MEM,1000, CACHE_WAIT);

                    /* read, this data should match data_2 */
                    Data_4 = *(Uint32*)EMIF_MEM;
                    if(Data_2 != Data_4) {
                        printf("CACHE: Write back invalidate");
                        printf(" L2 cache Failed.\n");
                        cacheExampleFail++;
                    }
                    else {
                        printf("CACHE: Write back invalidate");
                        printf("L2 cache Passed.\n");
                    }
                    break;

                /* Test #6 wb */
                case 6:
                    /* Writeback L2 cache */
                    CACHE_wbL2((Uint32*)EMIF_MEM, 4, CACHE_WAIT);

                    /* read, this data should match data_2 */
                    Data_4 = *(Uint32*)EMIF_MEM;
                    if(Data_2 != Data_4) {
                        printf("CACHE: Write back L2 cache Failed.\n");
                        cacheExampleFail++;
                    }
                    else {
                        printf("CACHE: Write back L2 cache Passed.\n");
                    }
                    break;

                /* Test #6 wb */
                case 7:
                    /* Set L2 cache in normal mode */
                    CACHE_setL2Mode(CACHE_L2_NORMAL);

                    /* read, this data should match data_3 */
                    Data_4 = *(Uint32*)EMIF_MEM;
                    if(Data_3 != Data_4) {
                        printf("CACHE: L2 Normal Mode Failed.\n");
                        cacheExampleFail++;
                    }
                    else {
                        printf("CACHE: L2 Normal Mode Passed.\n");
                    }
                    break;

                case 8:
                    /* Writeback invalidate the cache */
                    CACHE_wbInvAllL2(CACHE_WAIT);

                    /* freeze the cache */
                    if(CACHE_setL2Mode(CACHE_L2_FREEZE) != CACHE_L2_NORMAL)
                    {}

                    /* read, this would normally allocate space in the cache */
                    temp = *(Uint32*)EMIF_MEM;
                    *(Uint32*)EMIF_MEM = 0xABCDABCD;

                    /* write to external memory */
                    *(Uint32*)EMIF_MEM =0xA5A5A5A5;
                    if(CACHE_setL2Mode(CACHE_L2_NORMAL) != CACHE_L2_FREEZE)
                    {}
                    Data_4 = *(Uint32*)EMIF_MEM;

                    /* if the cache was frozen then the Data_4 should equal */
                    if(Data_4 != 0xA5A5A5A5) {
                        printf("CACHE: Freeze operation on Cache Failed.\n");
                        cacheExampleFail++;
                    }
                    else {
                        printf("CACHE: Freeze operation on Cache Passed.\n");
                    }
                    break;

                default:
                    break;
            }

            /* L2 is turned off */
            CACHE_setL2Size(CACHE_0KCACHE);
            /* disable caching for CE_00 */
            *(Uint32*) 0x01848200 = 0;
        }
    }

    subExampleEndPrint();

    /* Log the result.*/
    if (cacheExampleFail > 0) {
        printf ("\nCACHE: Example on L2 cache FAILED. \
                [Tests Failed: %ld]\n", cacheExampleFail);
    }
    else {
        printf ("\nCACHE: Example on L2 cache PASSED\n");
    }

    exampleEndPrint();

    cacheExampleFail = 0;

    /*========================================================================*
     *                    Test L1D                                            *
     *========================================================================*/

    /* Array used for setting L1 cache size */
    localSize[0] = (CACHE_L2Size)CACHE_L1_4KCACHE;
    localSize[1] = (CACHE_L2Size)CACHE_L1_8KCACHE;
    localSize[2] = (CACHE_L2Size)CACHE_L1_16KCACHE;
    localSize[3] = (CACHE_L2Size)CACHE_L1_32KCACHE;


    /* This loop is used to display the L1 cache size while performing different
     * operations on different L1 cache sizes
     */
    for(loopCount = 0; loopCount <4; loopCount++) {
        switch(loopCount) {
            case 0:
                printf("\n");
                subExampleEndPrint();
                printf("CACHE: Performing operations on 4K L1 cache.\n");
                subExampleEndPrint();
                break;

            case 1:
                printf("\n");
                subExampleEndPrint();
                printf("CACHE: Performing operations on 8K L1 cache.\n");
                subExampleEndPrint();
                break;
            case 2:
                printf("\n");
                subExampleEndPrint();
                printf("CACHE: Performing operations on 16K L1 cache.\n");
                subExampleEndPrint();
                break;

            case 3:
                printf("\n");
                subExampleEndPrint();
                printf("CACHE: Performing operations on 32K L1 cache.\n");
                subExampleEndPrint();
                break;

            default:
                    break;
        }

        for(count = 1; count <= 6; count++) {
            /* set a test memory location */
            *(Uint32*)EMIF_MEM = 0xABCDABCD;
            Data_2 = 0xa5a5a5a5;

            /* set Mar bit for CACHE_MEM */
            CACHE_enableCaching(CACHE_EMIFA_CE00);

            /* turn on the L1D cache. */
            CACHE_setL1dSize( (CACHE_L1Size)localSize[loopCount]);

            /* read (allocate into cache) */
            Data_1 = *(Uint32*)EMIF_MEM;

            /* write (dirty the cache line) */
            *(Uint32*)EMIF_MEM = Data_2;

            /* Read the data back (make sure it matches Data_2) */
            Data_3 = *(Uint32*)EMIF_MEM;

            if(Data_2 != Data_3)
                printf("CACHE: Data read from cache is not same as \
                        data written.\n");

            switch(count) {
                /* test #1 - invAllL1D */
                case 1:
                    /* Invalidate all L1D cache */
                    CACHE_invAllL1d(CACHE_WAIT);

                    /* read, this data should match data_1 */
                    Data_4 = *(Uint32*)EMIF_MEM;
                    if(Data_1 != Data_4) {
                        printf("CACHE: Invalidate All L1D cache Failed.\n");
                        cacheExampleFail++;
                    }
                    else {
                        printf("CACHE: Invalidate All L1D cache Passed.\n");
                    }
                    break;

                /* Test #2 - wbInvAllL1D */
                case 2:
                    /* Writeback invalidate all L1D cache */
                    CACHE_wbInvAllL1d(CACHE_WAIT);

                    /* read, this data should match data_2 */
                    Data_4 = *(Uint32*)EMIF_MEM;
                    if(Data_2 != Data_4) {
                        printf("CACHE: Write back invalidate All");
                        printf("L1D cache Failed.\n");
                        cacheExampleFail++;
                    }
                    else {
                        printf("CACHE: Write back invalidate All");
                        printf(" L1D cache Passed.\n");
                    }
                    break;

                /* Test #3 wbAll */
                case 3:
                    /* Writeback all L1D cache */
                    CACHE_wbAllL1d(CACHE_WAIT);

                    /* read, this data should match data_2 */
                    Data_4 = *(Uint32*)EMIF_MEM;
                    if(Data_2 != Data_4) {
                        printf("CACHE: Write back All L1D cache Failed.\n");
                        cacheExampleFail++;
                    }
                    else {
                        printf("CACHE: Write back All L1D cache Passed.\n");
                    }

                    break;

                /* test #4 - invL1D */
                case 4:
                    /* Invalidate L1D cache */
                    CACHE_invL1d((Uint32*)EMIF_MEM,4, CACHE_WAIT);

                    /* read, this data should match data_1 */
                    Data_4 = *(Uint32*)EMIF_MEM;
                    if(Data_1 != Data_4) {
                        printf("CACHE: Invalidate L1D cache Failed.\n");
                        cacheExampleFail++;
                    }
                    else {
                        printf("CACHE: Invalidate L1D cache Passed.\n");
                    }
                    break;

                /* Test #5 - wbInv */
                case 5:
                    /* Writeback invalidate L1D cache */
                    CACHE_wbInvL1d((Uint32*)EMIF_MEM,1000, CACHE_WAIT);

                    /* read, this data should match data_2 */
                    Data_4 = *(Uint32*)EMIF_MEM;
                    if(Data_2 != Data_4) {
                        printf("CACHE: Write back invalidate");
                        printf("L1D cache Failed.\n");
                        cacheExampleFail++;
                    }
                    else {
                        printf("CACHE: Write back invalidate");
                        printf("L1D cache Passed\n");
                    }
                    break;

                /* Test #6 wb */
                case 6:
                    /* Writeback L1D cache */
                    CACHE_wbL1d((Uint32*)EMIF_MEM,1000, CACHE_WAIT);

                    /* read, this data should match data_2 */
                    Data_4 = *(Uint32*)EMIF_MEM;
                    if(Data_2 != Data_4) {
                        printf("CACHE: Write back L1D cache Failed.\n");
                        cacheExampleFail++;
                    }
                    else {
                        printf("CACHE: Write back L1D cache Passed.\n");
                    }
                    break;

                default:
                    break;
            }

            /* L1D is turned off */
            CACHE_setL1dSize(CACHE_L1_0KCACHE);

            /* disable caching for CE_00 */
            *(Uint32*) 0x01848200 = 0;
        }
    }

    subExampleEndPrint();

    /* Log the result.*/
    if (cacheExampleFail > 0) {
        printf ("\nCACHE: Example on L1 cache FAILED. \
            [Tests Failed: %ld]\n", cacheExampleFail);
    }
    else {
        printf ("\nCACHE: Example on L1 cache PASSED\n");
    }

    printf ("\n\nCACHE Disable-Enable Test\n");
    
    /* set a test memory location */
    *(Uint32*)EMIF_MEM = 0xDEADBEEF;

    /* Initialize */
    Data_2 = 0xFEEDDEAD;

    /* Set Mar bit for EMIF_MEM */
    CACHE_enableCaching(CACHE_EMIFA_CE00);

    /* Turn on the L2 cache. */
    CACHE_setL2Size(CACHE_32KCACHE);

    /* read (allocate into cache) */
    Data_1 = *(Uint32*)EMIF_MEM;

    /* Write (dirty the cache line) */
    *(Uint32*)EMIF_MEM = Data_2;

    /* Read the data back (make sure it matches Data_2 */
    Data_3 = *(Uint32*)EMIF_MEM;
    if(Data_2 != Data_3)
        printf("CACHE: Data read from memory is not same as data written.\n");

    /* We shall now do a cache invalidate. Since caching is enabled right now
     * all the dirty lines "Data_2" above will be thrown out. */
    CACHE_invAllL2(CACHE_WAIT);

    /* Read the data from the external memory again and since the cache
     * contents have been invalidated; the value read should be the same
     * as the original data contents. */
    Data_4 = *(Uint32*)EMIF_MEM;
    if(Data_1 != Data_4) 
        printf("CACHE: Cache Enable Test FAILED\n");
    else
        printf("CACHE: Cache Enable Test Passed.\n");

    /* Set a test memory location */
    *(Uint32*)EMIF_MEM = 0xDEADBEEF;

    /* Initialize */
    Data_2 = 0xFEEDDEAD;

    /* Disable the caching support for the EMIF Memory region. */
    CACHE_disableCaching(CACHE_EMIFA_CE00);
    
    /* Invalidate the cache contents; if not then even though no more
     * caching is done the contents of the cache are still looked into. */ 
    CACHE_invAllL2(CACHE_WAIT);

    /* read (allocate into cache) */
    Data_1 = *(Uint32*)EMIF_MEM;

    /* Write (dirty the cache line) */
    *(Uint32*)EMIF_MEM = Data_2;

    /* Read the data back (make sure it matches Data_2) */
    Data_3 = *(Uint32*)EMIF_MEM;
    if(Data_2 != Data_3)
        printf("CACHE: Data read from memory is not same as data written.\n");

    /* We shall now do a cache invalidate. Since Caching is disabled this 
     * operation should do nothing.  */
    CACHE_invAllL2(CACHE_WAIT);

    /* Read the data from the external memory and this time the contents of
     * should be the same as the "dirty" line; since there was no cache 
     * in the picture and the invalidate above was a NOP. */
    Data_4 = *(Uint32*)EMIF_MEM;
    if(Data_2 != Data_4) 
        printf("CACHE: Cache Disable Test FAILED\n");
    else
        printf("CACHE: Cache Disable Test Passed.\n");

    exampleEndPrint();
}

void subExampleEndPrint (void)
{
    printf ("*************************************************************\n");
}

void exampleEndPrint (void)
{
    printf ("=============================================================\n");
}

