/*  ===========================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006
 *
 *   Use of this software is controlled by the terms and conditions found in 
 *   the license agreement under which this software has been supplied.
 *   ==========================================================================
 */
 
 /** ===========================================================================
 *   @file  cic_example1.c
 *
 *   @path  $(CSLPATH)\example\cic\cic_example1\src
 *
 *   @desc  Example of CIC
 *
 * ============================================================================
 *   @n <b> Example Description </b>
 *   @n This example illustrates the opening of an event handle, 
 *      associating a handler with it, enabling it for 60 events starting from 
 *      4 to 63. This example,
 *          1. Initalizes the cic CSL.
 *          2. Opens event 20 with evtCtl vector line 0,
 *          3. Initalizes the intc CSL required with proper ISR
 *          4. Opens event 80 with vector line 4.             
 *          3. Enable event 84 in INTC.
 *          4. Enable event 20 in CIC.
 *          4. Manually set event 20 in CIC.
 *          5. Prints CIC event mapped to INTC has occured.
 *
 * =============================================================================
 *      
 *   <b> Procedure to run the example </b>
 *   @verbatim
 *      1. Configure the CCS setup to work with the emulator being used
 *      2. Please refer CCS manual for setup configuration and loading 
 *         proper GEL file
 *      3. Launch CCS window
 *      4. Open project cic_example1.pjt
 *      5. Build the project and load the .out file of the project.
 *          
 *   @endverbatim
 *
 */

/* ============================================================================
 *  Revision History
 *  ===============
 *  2-Feb-2006 pd File Created
 * ============================================================================
 */

#include <stdio.h>
#include <csl_intc.h>
#include <csl_intcAux.h>
#include <csl_cic.h>
#include <csl_cicAux.h>

/* Intc variables declaration */
CSL_IntcContext                 context; 
CSL_IntcObj                     intcObj0;
CSL_IntcHandle                  hIntc0;
CSL_IntcEventHandlerRecord      EventHandler[30];
CSL_IntcGlobalEnableState       state;
CSL_IntcEventHandlerRecord      EventRecord;

CSL_CicContext                  context1;
CSL_CicHandle                   hCic;
CSL_CicObj                      cicObj; 


/*  Function forwards */
void eventHandler(CSL_IntcEventId* event);
void cic_example1 (void);
void enableRAC(void);

/* Global variable declaration */
Uint32  evtNum = 0;

/*
 * ============================================================================
 *
 *   @func   main
 *
 *   @desc
 *     Application that calls example function
 * ============================================================================
*/
void main (void)
{
    printf ("Running Interrupt Example\n");
    
    cic_example1 ();

    return;
    
}

/*
 * ============================================================================
 *   @func cic_example1
 *   
 *   @arg
 *      NONE
 *
 *   @desc
 *     This checks functionalities of the cic CSL.It implements following steps
 *          1. Initializes the cic module 
 *          2. Opens the event 32 with interrupt vector line 0 
 *          3. Initializes the INTC module 
 *          4. Opens the event 80 with interrupt vector line 4 
 *          5. Enable the event 32 in CIC.
 *          6. Enable the event 80 in INTC.
 *          7. Sets manually the event 32 in CIC.
 *          8. Prints CIC event mapped to INTC has occured.
 *   
 *   @return
 *      NONE
 *
 * =============================================================================
 */

void cic_example1 (void)
{   
    /* Obj for combined events */
	CSL_Status          intStat;
    CSL_IntcParam       vectId;
    CSL_CicParam        param;


	intStat = CSL_cicInit(&context1);
    if (intStat != CSL_SOK) {
        printf ("CIC: Initialization... Failed.\n");
        printf ("\tReason: CSL_CicInit failed. [status = 0x%x].\n", intStat);
    }
    else {
        printf ("CIC: Initialization... Passed.\n");
    }

    // Opening a handle for the CSL_CIC_EVENTID_FSEVT18 at Cic Output Event ID 0
    param.eventId = CSL_CIC_EVENTID_FSEVT18;
    param.ectlEvtId = CSL_CIC_ECTL_EVT0;
    
    hCic = CSL_cicOpen(&cicObj, CSL_CIC_0, &param , &intStat);
    if ((hCic == NULL) || (intStat != CSL_SOK)) {
            printf ("CIC: Open... Failed.\n");
            printf ("\tReason: Error opening the instance ");
            printf ("[status = 0x%x, hCic = 0x%x]\n", intStat, hCic);            
    }
	else
        printf ("CIC: Open... Passed.\n");

    /* INTC initialization strats from here */
    context.numEvtEntries = 1; 
    context.eventhandlerRecord = EventHandler;
    
    /* Initializes the CPU vector table, dispatchr */
    intStat = CSL_intcInit(&context);
    if (intStat != CSL_SOK) {
        printf ("INTR: Initialization... Failed.\n");
        printf ("\tReason: CSL_intcInit failed. [status = 0x%x].\n", intStat);
    }
    else {
        printf ("INTR: Initialization... Passed.\n");
    }

    /* Enable NMIs */
    intStat = CSL_intcGlobalNmiEnable();
    if (intStat != CSL_SOK) {
        printf ("INTR: Global NMI Enable ... Failed.\n");
        printf ("\tReason: CSL_intcGlobalNmiEnable failed. [status = 0x%x]\n", intStat);
    }
    else {
        printf ("INTR: Global NMI Enable... Passed.\n");
    }

    /* Enable Global Interrupts */
    intStat = CSL_intcGlobalEnable(&state);
    if (intStat != CSL_SOK) {
        printf ("INTR: Global Enable ... Failed.\n");
        printf ("\tReason: CSL_intcGlobalEnable failed. [status = 0x%x].\n", intStat);
    }
    else {
        printf ("INTR: Global Enable... Passed.\n");
    }
    
    /* Opening a handle for CIC out 0 onto CPU vector 4 */
    vectId = CSL_INTC_VECTID_4;
    hIntc0 = CSL_intcOpen (&intcObj0, CSL_INTC_EVENTID_CIC_EVT0, &vectId , NULL);
    if ((hIntc0 == NULL) || (intStat != CSL_SOK)) {
            printf ("INTR: Open... Failed.\n");
            printf ("\tReason: Error opening the instance ");
            printf ("[status = 0x%x, hIntc = 0x%x]\n", intStat, hIntc0);
            
    }
    else
        printf ("INTR: Open... Passed.\n");

	EventRecord.handler = (CSL_IntcEventHandler)&eventHandler;
    EventRecord.arg = hIntc0;
    CSL_intcPlugEventHandler(hIntc0,&EventRecord); 
             
	
	intStat = CSL_cicHwControl(hCic,CSL_CIC_CMD_EVTENABLE, NULL);
    if(intStat != CSL_SOK) {
    	printf("CIC: HwControl to enable event ... Failed.\n");
    }
    else
    	printf("CIC: HwControl to enable event ... Passed.\n");
     
    intStat = CSL_intcHwControl(hIntc0,CSL_INTC_CMD_EVTENABLE,NULL);
    if(intStat != CSL_SOK) {
    	printf("INTR: HwControl to enable event ... Failed.\n");
    }
    else
    	printf("INTR: HwControl to enable event ... Passed.\n");

	/* Set events manually */
    intStat = CSL_cicHwControl(hCic,CSL_CIC_CMD_EVTSET,NULL);
    if(intStat != CSL_SOK) {
    	printf("CIC: HwControl to set an event  ... Failed.\n");
    }
    else
    	printf("CIC: HwControl to set an event ... Passed.\n");
           
    while(!evtNum);
          
    intStat = CSL_intcClose(hIntc0);
    if (intStat != CSL_SOK) {
        printf ("INTR: Close ... Failed.\n");
        printf ("\tReason: Intc Close failed. [status = 0x%x].\n", intStat);
    }
    else {
        printf ("INTR: Close ... Passed.\n");
    }
    
    intStat = CSL_cicClose(hCic);
    if (intStat != CSL_SOK) {
        printf ("CIC: Close ... Failed.\n");
        printf ("\tReason: Intc Close failed. [status = 0x%x].\n", intStat);
    }
    else {
        printf ("CIC: Close ... Passed.\n");
    }
        
}  
 
/*
 * =============================================================================
 *   @func   eventHandler
 *
 *   @desc
 *     This is the intc event handler
 *
 *   @arg    event
 *           Pointer to events
 *
 *
 *   @eg
 *      eventHandler ();
 * =============================================================================
 */
void eventHandler (
    CSL_IntcEventId     *event
)
{
    evtNum = 1;
    printf("INTR: CIC event %d mapped to INTC EVENT Handler for event:%d\n",\
            hCic->eventId, *event);
    CSL_intcEventClear (*event);
	CSL_cicEventClear (hCic);

}
