/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
 *   ===========================================================================
 */
/** ===========================================================================
 *   @file etb_example.c
 *
 *   @path  $(CSLPATH)\example\etb\src
 *
 *   @desc This is a example of Application Managed On-Chip Capture of Trace 
 *          With Gated SPM output.
 *
 *  ============================================================================
 *   @n Target Platform: VDB
 *  ============================================================================
 *   @n <b> Example Description </b>
 *   @n This is an example of the CSL ETB.
 *   @verbatim
       1. Initialize DTF.
       2. Initialize ETB.
       3. Set trace clock i.e., cpu/3 
       4. Claim DTF ownership
       5. Enable DTF ownership
       6. Claim ownership for trace export block 
       7. Program Pin manager to have EMU pin allocation
       8. Enable ownership for trace export block to start generating trace test 
          pattern. 
       9. Disable formatting and put ETB formatter into bypass mode.
       10.Enable ETB data capture by writing ETB Control Register.
       11.Program trace control based on width
       12.Generate the specified trace test pattern
       13.change trace clock to cpu/6 
       14.Disable ETB data capture by writing ETB Control Register.
       15.Initialize the ETB RAM read pointer register with start_addr
       16.Read 1KB trace data out of ETB and log it into L2 memory
       17.Re-nable ETB data capture by writing ETB Control Register.
       18.Release DTF ownership
       19.Release ownership for trace export block

     @endverbatim
 *
 *=============================================================================
 *      
 *   @n <b> Procedure </b>
 *   @verbatim
      1. Configure the CCS setup
      2. Please refer CCS manual for setup configuration and loading 
         proper GEL file
      3. Launch CCS window
      4. Open project Etb_example.pjt
      5. Build the project and load the .out file of the project.

     @endverbatim
 *       
 *   
 * ============================================================================
 */

/* =============================================================================
 *  Revision History
 *  ===============
 *  13-Apr-2006 NG   File Created for basic test of ETB module.
 * =============================================================================
 */
/****************************************************************************/     
/* Include Files                                                            */     
/****************************************************************************/     
#include <csl_etb.h>
#include <csl_dtf.h>
#include <stdio.h>
#include <c6x.h>

/* Size of ETB RAM in 32-bit words */
#define ETB_RAM_SIZE    1024

/* PIN manager MMR address. */
#define PIN_MGR0   0x1BC0130
#define PIN_MGR1   0x1BC0134
#define PIN_MGR2   0x1BC0138 
#define PIN_MGR3   0x1BC0164 

/* Trace control register MMR address. */
#define TR_CNTL    0x1BC013C
#define TF_CNTL    0x1BC014C
#define TR_SYNC    0x1BC0140

/****************************************************************************/     
/* Global Variables                                                         */     
/****************************************************************************/     

/* Define a memory buffer to store the data read out of ETB RAM so that the host 
 * can read out and log to a file. The app code cannot directly log data 
 * into file since we don't have CIO support here as we do in the CCS debugger. 
 */
unsigned int etb_ram_data[ETB_RAM_SIZE];

/****************************************************************************/     
/* Static Functions                                                         */     
/****************************************************************************/     
static int config_gem_trace_pat(int pattern);

CSL_EtbHandle   hEtb;
CSL_DtfHandle   hDtf;
CSL_DtfObj      dtfObj;
CSL_EtbObj      etbObj;

void dtf_initialization();
void etb_initialization();

/* Pragmas used to create special linker section */
#pragma DATA_SECTION(etb_ram_data,".etb_ram_data");

/*
 * =============================================================================
 *   @func   main
 *
 *   @desc
 *     This is the main routine,which invokes the test scripts
 * =============================================================================
*/
void main (
    void
)
{
    unsigned pattern;

	pattern = 1;

    dtf_initialization();

    config_gem_trace_pat(pattern);

}

/* =============================================================================
 *   @func   config_gem_trace_pat
 *
 *   @desc
 *      This function sets up the GEM Pin Manager, trace control/format 
 *      registers to generate the specified test pattern and use DTF/ETB to 
 *      capture. This function also retrieve data from ETB and store them into 
 *      L2 mem for host to check.
 *
 *   @arg 
 *      pattern:  test pattern number (0~3)                                 
 *
 *   @return None
 *
 * =============================================================================
 */
int config_gem_trace_pat (
    int pattern
)
{
    CSL_Status    status;
    Uint32        response;
	Uint32        arg;
    int flow;
	volatile int i;

	/* trace clock is cpu/3 */
    trace_clk_on(3);

    /* claim DTF ownership, DTF_CNTL:AOWN=0, DTFFLUSH=0, DTFENABLE=0, 
     * SPMDISABLE=0, DTFOWN_ST=1 
     */
    status = CSL_dtfHwControl(hDtf, CSL_DTF_CMD_CLAIM_OWNERSHIP, NULL);

	/* check AOWN bit to see if DTF ownership is now owned by app code. */
    status = CSL_dtfGetHwStatus(hDtf, CSL_DTF_QUERY_AOWNERSHIP, &response);
    if (status != CSL_SOK) {
        printf ("DTF: Hardware setup... Failed.\n");
        printf ("\tReason: Error in hardware Setup\n");
        return 0;
    }
    else {
        if (response != CSL_DTF_DTFCR_AOWN_APP) {
            printf ("DTF: Application ownership .... Failed.\n");
            return 0;
        }
        else {
            printf ("DTF: Application ownership .... Passed.\n");
        }
    }

    /* enable DTF ownership, DTF_CNTL:AOWN=0, DTFFLUSH=0, DTFENABLE=0, 
     * SPMDISABLE=0, DTFOWN_ST=2 
     */
    status = CSL_dtfHwControl(hDtf, CSL_DTF_CMD_ENA_OWNERSHIP, NULL);

    /* enable DTF and disable SPM input DTF_CNTL: AOWN=0, DTFFLUSH=0, 
     * DTFENABLE=1, SPMDISABLE=1, DTFOWN_ST=10b 
     */
    arg = 1;
    status = CSL_dtfHwControl(hDtf, CSL_DTF_CMD_SET_DTFENABLE, &arg);
    status = CSL_dtfHwControl(hDtf, CSL_DTF_CMD_DIS_SPMDISABLE, NULL);

    /* Claim ownership for trace export block */
    TCU_CNTL = 0x1;  

    /* Program Pin manager to have EMU pin allocation as follows to conform to 
     * DTF spec 3.2.1 requirements 
     */
    /* EMU0=trace_data[19], EMU1=trace_data[18], ..., EMU19=trace_data[0],
     * EMU20=clock0, EMU21=clock1                                           
     */
    *((volatile unsigned int*)PIN_MGR0) = 0xCCDDEEFF;
    *((volatile unsigned int*)PIN_MGR1) = 0x8899AABB;
    *((volatile unsigned int*)PIN_MGR2) = 0x00556677;
    *((volatile unsigned int*)PIN_MGR3) = 0x00000000;

    /* Enable ownership for trace export block to start generating trace test 
     * pattern. 
     */
    TCU_CNTL |= 0x2;

    etb_initialization();

    /* ETB FIFO reset by writing 0 to ETB RAM Write Pointer Register. */
    arg = 0;
    status = CSL_etbHwControl(hEtb, CSL_ETB_CMD_SET_RAM_WR_POINTER, &arg);

    /* Disable formatting and put ETB formatter into bypass mode. */
    arg = 0;
    status = CSL_etbHwControl(hEtb, CSL_ETB_CMD_SET_ENAFORMATTING, &arg);
    arg = 0;
    status = CSL_etbHwControl(hEtb, CSL_ETB_CMD_SET_CONT_FORMATTING, &arg);

    /* Enable ETB data capture by writing ETB Control Register. */
    status = CSL_etbHwControl(hEtb, CSL_ETB_CMD_ENA_TRACE_CAPTURE, NULL);

	/* Put some delays in here. */
	do {
        status = CSL_etbGetHwStatus(hEtb, CSL_ETB_QUERY_TRACECAP_STATUS, 
                                    &response);
	} while (response != CSL_ETB_CTL_TRACECAPEN_ENABLE);

    /* Program trace control based on width */
    *((volatile unsigned int*)TR_CNTL) = 0x0001C07D; /* claim */

    /* Generate the specified trace test pattern */
    if (pattern == 0) {
        flow = 8;
	}
    else if (pattern == 1) {
        flow = 9;
    }
    else if (pattern == 2) {
        flow = 10;
    }
    else if (pattern == 3) {
        flow = 11;
    }

    /* TF_CNTL: width=20, flow=specified, Go=0, Active=0 */
    *((volatile unsigned int*)TF_CNTL) = (0x1400) | (flow); 

    /* TF_CNTL: width=20, flow=specified, Go=1, Active=0 */
    *((volatile unsigned int*)TF_CNTL) = (0x1400) | (flow) | 0x20; 

	/* Put some delay here so that we can test pattern would be captured for 
     * a short period of time. 
     */
	for (i = 0; i < 50; i++) {
		asm ("\tNOP");
	}

	/* change trace clock to be cpu/6 */
	trace_clk_on(6);

	/* Put some delay here so that we can test pattern would be captured for 
     * a short period of time. 
     */
	for (i = 0;i < 50; i++) {
		asm ("\tNOP");
	}

    /* Disable ETB data capture by writing ETB Control Register. */
    status = CSL_etbHwControl(hEtb, CSL_ETB_CMD_DIS_TRACE_CAPTURE, NULL);

	/* Put some delays in here. */
	do {
        status = CSL_etbGetHwStatus(hEtb, CSL_ETB_QUERY_TRACECAP_STATUS, 
                                    &response);
	} while (response != CSL_ETB_CTL_TRACECAPEN_DISABLE);

    /* Read ETB Status Register to poll for acquisition complete */
    do {
        status = CSL_etbGetHwStatus(hEtb, CSL_ETB_QUERY_ACQUISITION_COMPLETE, 
                                    &response);
    } while (response != CSL_ETB_STS_ACQ_COMPLETE); 

    /* Initialize the ETB RAM read pointer register with start_addr */
    status = CSL_etbHwControl(hEtb, CSL_ETB_CMD_SET_RAM_RD_POINTER, 0);

	 memset (&etb_ram_data, 0, sizeof(unsigned int [ETB_RAM_SIZE]));

    /* Now read 4KB trace data out of ETB and log it into L2 memory */
    i = 0;
	
    do {
        /* Read the ETB RAM read data register to retrieve trace data. 
         * This would cause the read pointer register value to auto-increment 
         */
        status = CSL_etbRead(hEtb, &etb_ram_data[i]);
		i++;

    }while (i < 1024);	
    printf("Read 1KB trace data from the RAM read data register into \
            etb_ram_data buffer \n");

    /* Re-nable ETB data capture by writing ETB Control Register. */
    status = CSL_etbHwControl(hEtb, CSL_ETB_CMD_ENA_TRACE_CAPTURE, NULL);

	/* Put some delays in here. */
	do {
        status = CSL_etbGetHwStatus(hEtb, CSL_ETB_QUERY_TRACECAP_STATUS, &response);
	} while (response != CSL_ETB_CTL_TRACECAPEN_ENABLE);

    /* release DTF ownership, DTF_CNTL:AOWN=0, DTFFLUSH=0, DTFENABLE=0, 
     * SPMDISABLE=0, DTFOWN_ST=0 
     */
    status = CSL_dtfHwControl(hDtf, CSL_DTF_CMD_ENA_AOWN_EMU, NULL);
    status = CSL_dtfHwControl(hDtf, CSL_DTF_CMD_FLUSH_DTFFLUSH, 0);
    status = CSL_dtfHwControl(hDtf, CSL_DTF_CMD_SET_DTFENABLE, 0);
    status = CSL_dtfHwControl(hDtf, CSL_DTF_CMD_ENA_SPMDISABLE, NULL);
    status = CSL_dtfHwControl(hDtf, CSL_DTF_CMD_RELEASE_OWNERSHIP, NULL);

    /* Release ownership for trace export block */
    TCU_CNTL = 0x0;

    printf("ETB: Functionality example completes\n");
    return 0;
}

/* =============================================================================
 *   @func   trace_clk_on
 *
 *   @desc
 *      This function Change trace clock frequency.
 *
 *   @arg    None
 *
 *   @return None
 *
 * =============================================================================
 */
int trace_clk_on (
    unsigned int div_val
)
{
	/* turn on trace clock from PLL here. */
	int plldiv_addr;
	int pllcmd_addr;
	int pllcmd_val;
    int plldiv_val;

	pllcmd_addr = 0x029A0138; //PLLCMD register
	plldiv_addr = 0x029A0184;// sysclk12
    
	/* PLLDIV register value: 
	bit 4:0 -- ratio = 00011b (/4), 11111b (/32)
	bit 15 -- DnEn = 1 */
	/* PLL divider value could be from 1 to 32. */
	if (div_val > 32)
		return -1;

	plldiv_val = (1<<15) | (div_val -1);
	*(unsigned int*)plldiv_addr = plldiv_val;

	/* PLLCMD register value:
	bit 1 -- GOSET */
	pllcmd_val = 0x1;
	*(unsigned int*)pllcmd_addr = pllcmd_val;

    return 0;

}

/* =============================================================================
 *   @func   dtf_initialization
 *
 *   @desc
 *      This function initilaizes etb and opens specified instance of dtf.
 *
 *   @arg    None
 *
 *   @return None
 *
 * =============================================================================
 */
void dtf_initialization (
    void
)
{
    CSL_Status     status;

    /* Initialize  DTF CSL module */
    status = CSL_dtfInit(NULL);
    if (status != CSL_SOK) {
        printf("DTF: Initialization... Failed.\n");
        printf("\tReason: CSL_dtfInit failed. [status = %d].\n", status);
        return;
    }
    else {
        printf("DTF: Initialization... Passed.\n");
    }

    /* Open DTF instance and verify */

    memset(&dtfObj, 0, sizeof(CSL_DtfObj));

    /* Open the CSL module for instance 0 */
    hDtf = CSL_dtfOpen(&dtfObj, (CSL_InstNum)CSL_DTF_0, NULL, &status);
    
    if ((hDtf == NULL) || (status != CSL_SOK)) {
        printf("DTF: Opening instance0... Failed.\n");
        printf("\tReason: Error opening the instance0. \
                [status = 0x%x, hDtf = 0x%x]\n", status, hDtf);
        return;
    }
}

/* =============================================================================
 *   @func   etb_initialization
 *
 *   @desc
 *      This function initilaizes etb and opens specified instance of etb.
 *
 *   @arg    None
 *
 *   @return None
 *
 * =============================================================================
 */
void etb_initialization (
    void
)
{
    CSL_Status     status;

    /* Initialize  ETB CSL module */
    status = CSL_etbInit(NULL);
    if (status != CSL_SOK) {
        printf("ETB: Initialization... Failed.\n");
        printf("\tReason: CSL_etbInit failed. [status = %d].\n", status);
        return;
    }
    else {
        printf("ETB: Initialization... Passed.\n");
    }

    /* Open ETB instance and verify */

    memset(&etbObj, 0, sizeof(CSL_EtbObj));

    /* Open the CSL module for instance 0 */
    hEtb = CSL_etbOpen(&etbObj, (CSL_InstNum)CSL_ETB_0, NULL, &status);
    
    if ((hEtb == NULL) || (status != CSL_SOK)) {
        printf("ETB: Opening instance0... Failed.\n");
        printf("\tReason: Error opening the instance0. \
                [status = 0x%x, hEtb = 0x%x]\n", status, hEtb);
        return;
    }
}

