/* ============================================================================
 * Copyright (c) Texas Instruments Inc 2008
 *
 * Use of this software is controlled by the terms and conditions found in the
 * license agreement under which this software has been supplied.
 * ============================================================================
 */

/** ===========================================================================
 *   @file Srio_evm_dio_exampleSrc.c
 *
 *   @path  $(CSLPATH)\example\srio_evm_dio_example\src
 *
 *   @desc This is a simple external lookback example of usage of CSL APIs 
 *         of the SRIO module.
 *
 *  ============================================================================
 *   @n Target Platform: EVM Faraday #1 
 *  ============================================================================
 *
 * ============================================================================
 *   @n <b> Example Description </b>
 *   @n This is an example of the CSL SRIO usage for a LSU port.
 *   @verbatim
       1. Setup SRIO module for external loop back transfer on LSU1.
       2. Setup the destination and source data areas.
       3. Configure LSU1 registers.
       4. Start transfer.
       5. Wait for completion of the transfer.
       6. Verify the destination contains the expected data.

     @endverbatim
 *
 *=============================================================================
 *      
 *   @n <b> Procedure </b>
 *   @verbatim
      1. Configure the CCS setup
      2. Please refer CCS manual for setup configuration and loading 
         proper GEL file
      3. Launch CCS window
      4. Open project Srio_evm_dio_exampleSrc.pjt
      5. Build the project and load the Srio_evm_dio_exampleSrc.out file of the project.

     @endverbatim
 *       
 *   
 * ============================================================================
 */

/* ============================================================================
 * Revision History
 * ===============
 * 4-Sept-2008  File created
 * ============================================================================
 */
#include <csl_srio.h>
#include <csl_srioAux.h>
#include <csl_psc.h>
#include <soc.h>
#include <stdio.h>

/***********************************************************************
 ************************** LOCAL Definitions **************************
 ***********************************************************************/

/* Device Identifiers (8bit and 16bit) for the Source Application. */
#define LARGE_DEV_ID         0xBEEF
#define SMALL_DEV_ID         0xAB

/* Device Identifier (16bit) for the Target Application. */
#define DST_LARGE_DEV_ID     0xFACE

/* Definition for the NWRITE RIO Command. */
#define SRIO_PKT_TYPE_NWRITE 0x54

/* Length of data being transferred. */
#define TRANSFER_SIZE        256

/* Destination Address of the Target where the data will be sent to. */
#define DST                  ((Uint8 *)0x1088d000)

/* SRIO Port Number on which the test will execute. */
#define SRIO_PORT_NUMBER     1

/************************************************************************
 ************************** GLOBAL Definitions **************************
 ************************************************************************/

/* Global Source Buffer which is sent via DIO to the destination address */
static Uint8 src[TRANSFER_SIZE];

/* Global SRIO Hardware Setup used to initialized the SRIO IP Block. */
static CSL_SrioHwSetup setup = CSL_SRIO_HWSETUP_DEFAULTS;

/************************************************************************
 **************************** Source Functions **************************
 ************************************************************************/

/*
 * ============================================================================
 *   @func   delay
 *
 *   @desc
 *     This routine is used to spin in a delay loop for delay iterations
 * ============================================================================
 */
static void delay(Uint32 delay)
{
    volatile Uint32 i, n;

    n = 0;
    for (i = 0; i < delay; i++)
    {
        n = n + 1;
    }
}

/*
 * ============================================================================
 *   @func   enableSRIO
 *
 *   @desc
 *     This routine is used to power up the SRIO IP Module.
 * ============================================================================
 */
static void enableSRIO()
{
    CSL_PscObj pscObj;
	CSL_PscHandle hPsc;
	CSL_Status status;
	CSL_PscPowerDomain pwrDmn = CSL_PSC_PWRDMN_PROXYB;
	CSL_PscPeripherals module = CSL_PSC_MODULE_SRIO;
 	CSL_PscPwrDmnTransState response;

	memset(&pscObj, 0, sizeof(CSL_PscObj));      

    /* Init PSC module */
	CSL_pscInit(NULL);

    /* Open PSC module to get a Handle */
	hPsc = CSL_pscOpen(&pscObj, CSL_PSC, NULL, &status);

    /* Enable power domain for SRIO */
	CSL_pscHwControl(hPsc, CSL_PSC_CMD_ENABLE_PWRDMN, &pwrDmn);
    /* Enable clock for the specified module SRIO */
	CSL_pscHwControl(hPsc, CSL_PSC_CMD_ENABLE_MODULE, &module);
	/*Enable SRIO Power domain GO transition */
	CSL_pscHwControl(hPsc, CSL_PSC_CMD_PWRDMN_TRNS, &pwrDmn);

	response.pwrDmn = CSL_PSC_PWRDMN_PROXYB;
    response.status = 0x0; 
	do{
    	CSL_pscGetHwStatus(hPsc, CSL_PSC_QUERY_PWRDMN_TRANS_STAT, (void *)&response);
		(response.status) &= (1 << CSL_PSC_PWRDMN_PROXYB);
	}while((response.status) != 0x0);
	
    /* Close the PSC module */
	CSL_pscClose(hPsc);
}

/*
 * ============================================================================
 *   @func   srio_Create_Setup
 *
 *   @desc
 *     This routine sets up a structure with the required setup to initialize
 *     the SRIO IP block.
 * ============================================================================
 */
static void srio_Create_Setup (CSL_SrioHwSetup *pSetup,int bootcomplete)
{
    Uint32 index;
    
    /* Peripheral enable */
    pSetup->perEn = 1;

    /* While in shutdown, put memories in sleep mode */
    pSetup->periCntlSetup.swMemSleepOverride = 1;
    
    /* Disable loopback operation 
     * NOTE: External lookback operation is enabled 
     */
    pSetup->periCntlSetup.loopback = 0;
    
    /* Boot process is over (complete) */
    pSetup->periCntlSetup.bootComplete = bootcomplete;
    
    /* Process TX requests of priority 2, when credit is 1 or greater */
    pSetup->periCntlSetup.txPriority2Wm = CSL_SRIO_TX_PRIORITY_WM_1;
    
    /* Process TX requests of priority 1, when credit is 1 or greater */
    pSetup->periCntlSetup.txPriority1Wm = CSL_SRIO_TX_PRIORITY_WM_2;

    /* Process TX requests of priority 0, when credit is 1 or greater */
    pSetup->periCntlSetup.txPriority0Wm = CSL_SRIO_TX_PRIORITY_WM_3;

    /* Set internal bus priority to 4 */
    pSetup->periCntlSetup.busTransPriority = CSL_SRIO_BUS_TRANS_PRIORITY_4;
    
    /* UDI buffers are port based not proirity based */
    pSetup->periCntlSetup.bufferMode = CSL_SRIO_1X_MODE_PORT;
    
    /* VBUS clock is of 333 MHz  from TPCC document section 2.3 */
    pSetup->periCntlSetup.prescalar = CSL_SRIO_CLK_PRESCALE_1;
    
    /* Enable port 0 PLL */
    pSetup->periCntlSetup.pllEn = 0/*CSL_SRIO_PLL1_ENABLE*/; 

    /* Enable clocks to all domains */
    pSetup->gblEn = 1;

    /* Enable clock in all the domains */
    for (index=0; index<9; index++)
        pSetup->blkEn[index] = 1;

    /* 8-bit id is 0xAB and 16-bit id is 0xBEEF */
    pSetup->deviceId1 = CSL_FMK(SRIO_BASE_ID_BASE_DEVICEID, SMALL_DEV_ID) | 
                        CSL_FMK(SRIO_BASE_ID_LARGE_BASE_DEVICEID, LARGE_DEV_ID);

    /* 8-bit id is 0xAB and 16-bit id is 0xBEEF for multi-cast*/
    pSetup->deviceId2 = CSL_FMK(SRIO_BASE_ID_BASE_DEVICEID, SMALL_DEV_ID) | 
                        CSL_FMK(SRIO_BASE_ID_LARGE_BASE_DEVICEID, LARGE_DEV_ID);

	/* SERDES PLL configuration for channel 0 */
	pSetup->serDesPllCfg.pllEnable      = TRUE;
	pSetup->serDesPllCfg.pllMplyFactor  = CSL_SRIO_SERDES_PLL_MPLY_BY_10;
	pSetup->serDesPllCfg.loopBandwidth  = CSL_SRIO_SERDES_LOOP_BANDWIDTH_FREQ_DEP;

    /* SERDES RX channel 0 enable */
    pSetup->serDesRxChannelCfg[0].enRx              = TRUE; 
    pSetup->serDesRxChannelCfg[0].symAlign          = CSL_SRIO_SERDES_SYM_ALIGN_COMMA; 
	pSetup->serDesRxChannelCfg[0].los               = CSL_SRIO_SERDES_LOS_DET_DISABLE;
	pSetup->serDesRxChannelCfg[0].clockDataRecovery = 0x00;  /* first order */
	pSetup->serDesRxChannelCfg[0].equalizer         = 0x01;
	pSetup->serDesRxChannelCfg[0].termination       = CSL_SRIO_SERDES_TERMINATION_0_8_VDDT;
    pSetup->serDesRxChannelCfg[0].busWidth          = CSL_SRIO_SERDES_BUS_WIDTH_10_BIT;
    pSetup->serDesRxChannelCfg[0].rate              = CSL_SRIO_SERDES_RATE_HALF;

    /* SERDES TX channel 0 enable */
    pSetup->serDesTxChannelCfg[0].enTx              = TRUE;
    pSetup->serDesTxChannelCfg[0].commonMode        = CSL_SRIO_SERDES_COMMON_MODE_RAISED;
    pSetup->serDesTxChannelCfg[0].outputSwing       = CSL_SRIO_SERDES_SWING_AMPLITUDE_750;
    pSetup->serDesTxChannelCfg[0].rate              = CSL_SRIO_SERDES_RATE_HALF; 
	pSetup->serDesTxChannelCfg[0].enableFixedPhase  = FALSE; 
    
    /* SERDES RX channel 1 enable */
    pSetup->serDesRxChannelCfg[1].enRx              = TRUE; 
    pSetup->serDesRxChannelCfg[1].symAlign          = CSL_SRIO_SERDES_SYM_ALIGN_COMMA; 
	pSetup->serDesRxChannelCfg[1].los               = CSL_SRIO_SERDES_LOS_DET_DISABLE;
	pSetup->serDesRxChannelCfg[1].clockDataRecovery = 0x00;  /* first order */
	pSetup->serDesRxChannelCfg[1].equalizer         = 0x01;
	pSetup->serDesRxChannelCfg[1].termination       = CSL_SRIO_SERDES_TERMINATION_0_8_VDDT;
    pSetup->serDesRxChannelCfg[1].busWidth          = CSL_SRIO_SERDES_BUS_WIDTH_10_BIT;
    pSetup->serDesRxChannelCfg[1].rate              = CSL_SRIO_SERDES_RATE_HALF;

    /* SERDES TX channel 1 enable */
    pSetup->serDesTxChannelCfg[1].enTx              = TRUE; 
    pSetup->serDesTxChannelCfg[1].commonMode        = CSL_SRIO_SERDES_COMMON_MODE_RAISED;
    pSetup->serDesTxChannelCfg[1].outputSwing       = CSL_SRIO_SERDES_SWING_AMPLITUDE_750;
    pSetup->serDesTxChannelCfg[1].rate              = CSL_SRIO_SERDES_RATE_HALF; 
	pSetup->serDesTxChannelCfg[1].enableFixedPhase  = FALSE; 

    /* Select flow control ID length 16-bit */
    pSetup->flowCntlIdLen[0] = 1; 

    /* Destination ID of flow n, same ids as we are doing loopback */
    pSetup->flowCntlId[0] = LARGE_DEV_ID; 

    /* Sets the number of address bits generated by the PE as a source and 
     * processed by the PE as the target of an operation as 34 bits */
    pSetup->peLlAddrCtrl = CSL_SRIO_ADDR_SELECT_34BIT;

    /* Base device configuration */
    pSetup->devIdSetup.smallTrBaseDevId  = SMALL_DEV_ID;
    pSetup->devIdSetup.largeTrBaseDevId  = LARGE_DEV_ID;
    pSetup->devIdSetup.hostBaseDevId     = LARGE_DEV_ID;

    /* Port General configuration */
    pSetup->portGenSetup.portLinkTimeout = 0xFFFFF;  /* 215 ms */
    pSetup->portGenSetup.portRespTimeout = 0xFFFFF;  /* 215 ms */
    pSetup->portGenSetup.hostEn          = 1;        /* Agent/Slave Device           */
    pSetup->portGenSetup.masterEn        = 1;        /* Device cannot issue requests */

    /* Port control configuration for PORT 0 */
    pSetup->portCntlSetup[0].portDis           = 0;         /* Do not disable Port 0 */
    pSetup->portCntlSetup[0].outPortEn         = 1;         /* Output on Port 0 enabled */
    pSetup->portCntlSetup[0].inPortEn          = 1;         /* Input on Port 0 enabled */
    pSetup->portCntlSetup[0].portWidthOverride = CSL_SRIO_PORT_WIDTH_NO_OVERRIDE;
    pSetup->portCntlSetup[0].errCheckDis       = 0;         /* Err check enabled */
    pSetup->portCntlSetup[0].multicastRcvEn    = 0;         /* Multicast receive Disabled */
    pSetup->portCntlSetup[0].stopOnPortFailEn  = 0;         /* Dont Stop on fail */
    pSetup->portCntlSetup[0].dropPktEn         = 0;         /* Dont Drop PKT */
    pSetup->portCntlSetup[0].portLockoutEn     = 0;         /* Send any PKT */

    /* Port control configuration for PORT 1 */
    pSetup->portCntlSetup[1].portDis           = 0;        /* Do not disable Port 1 */
    pSetup->portCntlSetup[1].outPortEn         = 1;        /* Output on Port 1 enabled */
    pSetup->portCntlSetup[1].inPortEn          = 1;        /* Input on Port 1 enabled */
    pSetup->portCntlSetup[1].portWidthOverride = CSL_SRIO_PORT_WIDTH_NO_OVERRIDE;
    pSetup->portCntlSetup[1].errCheckDis       = 0;        /* Err check enabled */
    pSetup->portCntlSetup[1].multicastRcvEn    = 0;        /* Multicast receive Disabled. */
    pSetup->portCntlSetup[1].stopOnPortFailEn  = 0;        /* Dont Stop on fail */
    pSetup->portCntlSetup[1].dropPktEn         = 0;        /* Dont Drop PKT */
    pSetup->portCntlSetup[1].portLockoutEn     = 0;        /* Send any PKT */

    /* Enable all logical/transport errors */
    pSetup->lgclTransErrEn = CSL_SRIO_IO_ERR_RESP_ENABLE            |
                             CSL_SRIO_ILL_TRANS_DECODE_ENABLE       |
                             CSL_SRIO_ILL_TRANS_TARGET_ERR_ENABLE   |
                             CSL_SRIO_PKT_RESP_TIMEOUT_ENABLE       |
                             CSL_SRIO_UNSOLICITED_RESP_ENABLE       |
                             CSL_SRIO_UNSUPPORTED_TRANS_ENABLE;

    /* Enable all Port errors */    
    pSetup->portErrSetup[1].portErrRateEn =
                            CSL_SRIO_ERR_IMP_SPECIFIC_ENABLE            |
                            CSL_SRIO_CORRUPT_CNTL_SYM_ENABLE            |
                            CSL_SRIO_CNTL_SYM_UNEXPECTED_ACKID_ENABLE   |
                            CSL_SRIO_RCVD_PKT_NOT_ACCPT_ENABLE          |
                            CSL_SRIO_PKT_UNEXPECTED_ACKID_ENABLE        |
                            CSL_SRIO_RCVD_PKT_WITH_BAD_CRC_ENABLE       |
                            CSL_SRIO_RCVD_PKT_OVER_276B_ENABLE          |
                            CSL_SRIO_NON_OUTSTANDING_ACKID_ENABLE       |
                            CSL_SRIO_PROTOCOL_ERROR_ENABLE              |
                            CSL_SRIO_UNSOLICITED_ACK_CNTL_SYM_ENABLE    |
                            CSL_SRIO_LINK_TIMEOUT_ENABLE;

    /* Decrement error rate counter every second */ 
    pSetup->portErrSetup[1].prtErrRtBias = CSL_SRIO_ERR_RATE_BIAS_1S;

    /* Allow only 2 errors after error threshold is reached */ 
    pSetup->portErrSetup[1].portErrRtRec = CSL_SRIO_ERR_RATE_COUNT_2;

    /*  Port error setup */ 
    pSetup->portErrSetup[1].portErrRtFldThresh   = 10; /* Err threshold = 10 */
    pSetup->portErrSetup[1].portErrRtDegrdThresh = 10; /* Err degrade threshold = 10 */
    
    /* Configure the Port IP Mode (SP_IP_MODE) Register. 
     *  IDLE_ERR_DIS    - 0b0;  IDLE Error checking enabled
     *  TX_FIFO_BYP ASS - 0b0;  The TX_FIFO is operational
     *  PW_DIS          - 0b0;  Port-Write Error reporting is enabled
     *  TGT_ID_DIS      - 0b0;  packet accepted if DestID != BaseID
     *  SELF_RST        - 0b0;  Self reset disabled
     *  MLTC_EN         - 0b0;  Multicast-Event Interrupt Disable 
     *  RST_EN          - 0b0;  Reset Interrupt Disable 
     *  PW_EN           - 0b0;  Port-Write-In Interrupt Disable 
    */
    pSetup->portIpModeSet = 0x4D000000;

    /* Configure the SP_IP_PRESCALE register */
    pSetup->portIpPrescalar = 8; 

    /*  Port-Write Timer. The timer defines a period to repeat sending an error
     *  reporting Port-Write request for software assistance. The timer stopped 
     *  by software writing to the error detect registers 900 ms
     */
    pSetup->pwTimer = CSL_SRIO_PW_TIME_8;

    /* Port control independent error reporting enable. Macros can be ORed
     *  to get the value
     */

    /*
       TX_FLW        - 0;     Receive link flow control
       SOFT_REC      - 0;     Hardware controlled error recovery 
       FORCE_REINIT  - 0;     Reinitialization process NOT forced
       TRANS_MODE    - 01;    transfer mode - Store & Forward Mode 
       DEBUG         - 1;     Debug enable
       SEND_DBG_PKT  - 0;     Do not force a debug packet
       ILL_TRANS_EN  - 1;     Illegal Transfer Error reporting Enable
       MAX_RETRY_EN  - 1;     Max_retry_error report enable
       MAX_RETRY_THR - 0x01;  Maximum Retry Threshold Trigger
       IRQ_EN        - 1;     Interrupt error report Enable
    */ 
    pSetup->portCntlIndpEn[1] = 0x01A20180; 
}

/*
 * ============================================================================
 *   @func   test_write_cmd
 *
 *   @desc
 *      The function tests the WRITE command by sending data to the specified
 *      destination ID using Direct IO transfer.
 * ============================================================================
 */
static Int32 test_write_cmd (CSL_SrioHandle hSrio)
{
    volatile Uint32             index;
    volatile Uint8*             dst=DST;
    CSL_SrioDirectIO_ConfigXfr  lsu_conf;
    CSL_SrioPortData            response;
    CSL_SrioLsuCompStat         completionCode;
    Uint32                      intStatus;
    Int32                       counter = 1;

    /* Send out the data */
    while (counter < 11)
    {
        printf ("Debug: Starting Transfer %d on SRIO Port %d\n", counter, SRIO_PORT_NUMBER);

        /* Read the Port Error Status register. */
        response.index = SRIO_PORT_NUMBER;
        CSL_srioGetHwStatus (hSrio, CSL_SRIO_QUERY_SP_ERR_STAT, &response);

        /* Was there an error detected? */
        if ((response.data & ~0x2) != 0)
        {
            /* Error was detected; we need to clear this before we can proceed... */
            printf ("Error: Port %d has Input/Output Error... clearing\n", SRIO_PORT_NUMBER);

            /* Clearing the errors... */
            CSL_srioHwControl (hSrio, CSL_SRIO_CMD_SP_ERR_STAT_CLEAR, &response);
        }
    
        /* Setup the source and destination */
        for(index=0; index<TRANSFER_SIZE; index++) 
        {
	        src[index] = index;
	        dst[index] = 0;
	    }

        /* Create an LSU configuration */
        lsu_conf.srcNodeAddr           = (Uint32)&src[0];       /* Source address */
        lsu_conf.dstNodeAddr.addressHi = 0;
        lsu_conf.dstNodeAddr.addressLo = (Uint32)DST;           /* Destination address          */
        lsu_conf.byteCnt               = TRANSFER_SIZE;         /* Size of data being sent.     */
        lsu_conf.idSize                = 1;                     /* 16 bit device id             */
        lsu_conf.priority              = 2;                     /* PKT priority is 2            */
        lsu_conf.xambs                 = 0;                     /* Not an extended address      */
        lsu_conf.dstId                 = DST_LARGE_DEV_ID;      /* Destination ID               */
        lsu_conf.intrReq               = 1;                     /* Interrupts                   */
        lsu_conf.pktType               = SRIO_PKT_TYPE_NWRITE;  /* Write with no response       */
        lsu_conf.hopCount              = 0;                     /* Valid for maintainance pkt   */
        lsu_conf.doorbellInfo          = 0;                     /* Not a doorbell pkt           */
        lsu_conf.outPortId             = SRIO_PORT_NUMBER;      /* Tx on the specific SRIO Port.*/

        /* Configure the LSU and start transmission */
        CSL_srioLsuSetup (hSrio, &lsu_conf, 0);

        /* Check if the LSU will generate interrupts or not? */
        if (lsu_conf.intrReq == 0)
        {
            /* No Interrupts: Wait for the completion of transfer */
            response.index = 0;
            while (1)
            {
                /* Poll and check if the LSU has completed */
                CSL_srioGetHwStatus (hSrio, CSL_SRIO_QUERY_LSU_BSY_STAT, &response);

                /* Is the LSU not busy */
                if (response.data == 0)
                    break;
            }
        }
        else
        {
            /* Interrupt: Check for the LSU Interrupt Condition Status Register bits */
            while (1)
            {
                CSL_srioGetLsuIntrStat (hSrio, &intStatus);
                if (intStatus != 0)
                    break;
            }
        }

        /* Get the completion Code for the specific LSU. */
        completionCode.portNum = CSL_SRIO_PORT_0;
        CSL_srioLsuCompCodeStat(hSrio, &completionCode);
        if (completionCode.lsuCompCode == 0x0)
        {
            /* Successfull Transfer. */
            printf ("Debug: SRIO Transfer %d was completed...\n", counter);
        }
        else
        {
            /* Failed Transfer. */
            printf ("Error: SRIO Write Transfer %d has a completion code of %d\n", 
                     counter, completionCode.lsuCompCode);
            return -1;
        }

        /* Increment the counter.... */
        counter++;

        /* Delay for some time before we send the next data. */
        delay (1000000);
    }

    /* Test Passed... */
    return 0;
}

/*
 * ============================================================================
 *   @func   main
 *
 *   @desc
 *     This is the main routine of this program which invokes the different
 *     components to complete a simple LSU transfer.
 * ============================================================================
*/
void main (void)
{
    volatile Uint32     index;
    CSL_SrioContext     context;
    CSL_Status          status;
    CSL_SrioCntlSym     cntrlSym;
    CSL_SrioHandle      hSrio;
    CSL_SrioObj         srioObj;
    CSL_InstNum         srioNum = 0;  /* Instance number of the SRIO */
    CSL_SrioParam       srioParam;    
    CSL_SrioPortData    response;

    /* Enable SRIO in PSC control module */
    enableSRIO();
    
    /* Initialization and Open of the SRIO */
    status = CSL_srioInit (&context);
    hSrio = CSL_srioOpen (&srioObj, srioNum, &srioParam, &status);
    if (status != CSL_SOK) {
        printf("SRIO: ... Cannot open SRIO, failed\n");
        return;
    }

    // Enable SRIO and blocks
    hSrio->regs->GBL_EN = 0;
    hSrio->regs->BLK_ENABLE[0].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[1].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[2].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[3].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[4].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[5].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[6].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[7].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[8].BLK_EN = 0x0;
    
    // Disable PLL enable to disable the SerDes registers
    hSrio->regs->SERDES_CFG_CNTL[0] = 0x0; 
    hSrio->regs->SERDES_CFG_CNTL[1] = 0x0; 
    hSrio->regs->SERDES_CFG_CNTL[2] = 0x0; 
    hSrio->regs->SERDES_CFG_CNTL[3] = 0x0; 

    /* Create the setup structure */
    srio_Create_Setup (&setup, 1);
	
    /* Setup the SRIO with the selected setup in the last step */
    status = CSL_srioHwSetup (hSrio, &setup);
    if (status != CSL_SOK) {
       	printf("SRIO: ... Hardware setup, failed\n");
        return;
    }

    /* Make sure that the SRIO Port is Initialized. */
    printf ("Debug: Waiting for SRIO Port %d to be initialized\n", SRIO_PORT_NUMBER);

    /* Loop around till the PORT is initialized. */
    response.index = SRIO_PORT_NUMBER;
    do 
    {
 		delay(10000);  //check the response every 100 cpu clcok = 100 ns
		CSL_srioGetHwStatus (hSrio, CSL_SRIO_QUERY_SP_ERR_STAT, &response);
    }while(response.data & 0x1);

    /* Clear any errors... */
    cntrlSym.portNum = (CSL_SrioPortNum)SRIO_PORT_NUMBER;
    cntrlSym.stype0  = 0x2;
    cntrlSym.par0    = 0x0;
    cntrlSym.par1    = 0x1F;
    cntrlSym.stype1  = 0x4;
    cntrlSym.cmd     = 0x4;
    cntrlSym.emb     = 0x0;
    CSL_srioHwControl (hSrio, CSL_SRIO_CMD_CNTL_SYM_SET, &cntrlSym);

    /* Control comes here implies that the SRIO port has been initialized and can
     * now be used to send data. */
    printf ("Debug: SRIO Port %d has been initialized...\n", SRIO_PORT_NUMBER);

    /* Execute the WRITE commands and send data across to the Target. */
    if (test_write_cmd (hSrio) < 0)
    {
        printf ("Error: Write CMD Test Failed\n");
        while (1);
    }
    printf ("Debug: Write CMD Test Passed\n");

    /* Loop around forever.... */
    while (1);
}

