/* ============================================================================
 * Copyright (c) Texas Instruments Inc 2008
 *
 * Use of this software is controlled by the terms and conditions found in the
 * license agreement under which this software has been supplied.
 * ============================================================================
 */

/** ===========================================================================
 *   @file Srio_evm_dio_exampleTgt.c
 *
 *   @path  $(CSLPATH)\example\srio_evm_dio_example\src
 *
 *   @desc This is the target code for the external lookback example 
 *         of the SRIO module.  It should be run on the second Faraday.
 *         It take all RX packets and sends them out the TX link.
 *
 *  ============================================================================
 *   @n Target Platform: EVM Faraday #2 
 *  ============================================================================
 *
 * ============================================================================
 *   @n <b> Example Description </b>
 *   @n This is an example of the CSL SRIO usage for a LSU port.
 *   @verbatim
       1. Setup SRIO module for external loop back transfer on LSU1.
       2. Setup the destination and source data areas.
       3. Configure LSU1 registers.
       4. Start transfer.
       5. Wait for completion of the transfer.
       6. Verify the destination contains the expected data.

     @endverbatim
 *
 *=============================================================================
 *      
 *   @n <b> Procedure </b>
 *   @verbatim
      1. Configure the CCS setup
      2. Please refer CCS manual for setup configuration and loading 
         proper GEL file
      3. Launch CCS window
      4. Open project Srio_evm_dio_exampleTgt.pjt
      5. Build the project and load the Srio_evm_dio_exampleTgt.out file of the project.

     @endverbatim
 *       
 *   
 * ============================================================================
 */

/* ============================================================================
 * Revision History
 * ===============
 * 31-Aug-2006 NG  File created
 * ============================================================================
 */
#include <csl_srio.h>
#include <csl_psc.h>
#include <soc.h>
#include <stdio.h>
#include <csl_srioAux.h>

/***********************************************************************
 ************************** LOCAL Definitions **************************
 ***********************************************************************/

/* Device Identifiers (8bit and 16bit) for the Target Application. */
#define LARGE_DEV_ID            0xFACE
#define SMALL_DEV_ID            0xAC

/* Data Size which is being transferred */
#define TRANSFER_SIZE           256

/* Destination Address where the data will be sent to. */
#define DST                     ((Uint8 *)0x1088d000)

/* SRIO Port Number on which the test will execute. */
#define SRIO_PORT_NUMBER     1

/************************************************************************
 ************************** GLOBAL Definitions **************************
 ************************************************************************/

/* Global SRIO Hardware Setup used to initialized the SRIO IP Block. */
static CSL_SrioHwSetup setup = CSL_SRIO_HWSETUP_DEFAULTS;

/************************************************************************
 **************************** Target Functions **************************
 ************************************************************************/

/*
 * ============================================================================
 *   @func   delay
 *
 *   @desc
 *     This routine is used to spin in a delay loop for delay iterations
 * ============================================================================
 */
static void delay(Uint32 delay)
{
    volatile Uint32 i, n;

    n = 0;
    for (i = 0; i < delay; i++)
    {
        n = n + 1;
    }
}

/*
 * ============================================================================
 *   @func   enableSRIO
 *
 *   @desc
 *     This routine is used to power up the SRIO IP Module.
 * ============================================================================
 */
static void enableSRIO()
{
    CSL_PscObj pscObj;
	CSL_PscHandle hPsc;
	CSL_Status status;
	CSL_PscPowerDomain pwrDmn = CSL_PSC_PWRDMN_PROXYB;
	CSL_PscPeripherals module = CSL_PSC_MODULE_SRIO;
 	CSL_PscPwrDmnTransState response;

	memset(&pscObj, 0, sizeof(CSL_PscObj));      

    /* Init PSC module */
	CSL_pscInit(NULL);

    /* Open PSC module to get a Handle */
	hPsc = CSL_pscOpen(&pscObj, CSL_PSC, NULL, &status);

    /* Enable power domain for SRIO */
	CSL_pscHwControl(hPsc, CSL_PSC_CMD_ENABLE_PWRDMN, &pwrDmn);
    /* Enable clock for the specified module SRIO */
	CSL_pscHwControl(hPsc, CSL_PSC_CMD_ENABLE_MODULE, &module);
	/*Enable SRIO Power domain GO transition */
	CSL_pscHwControl(hPsc, CSL_PSC_CMD_PWRDMN_TRNS, &pwrDmn);

	response.pwrDmn = CSL_PSC_PWRDMN_PROXYB;
    response.status = 0x0; 
	do{
    	CSL_pscGetHwStatus(hPsc, CSL_PSC_QUERY_PWRDMN_TRANS_STAT, (void *)&response);
		(response.status) &= (1 << CSL_PSC_PWRDMN_PROXYB);
	}while((response.status) != 0x0);
	
    /* Close the PSC module */
	CSL_pscClose(hPsc);
}

/*
 * ============================================================================
 *   @func   srio_Create_Setup
 *
 *   @desc
 *     This routine sets up a structure with the required setup to initialize
 *     the SRIO IP block.
 * ============================================================================
 */
static void srio_Create_Setup (CSL_SrioHwSetup *pSetup,int bootcomplete)
{
    Uint32 index;
    
    /* Peripheral enable */
    pSetup->perEn = 1;

    /* While in shutdown, put memories in sleep mode */
    pSetup->periCntlSetup.swMemSleepOverride = 1;
    
    /* Disable loopback operation 
     * NOTE: External lookback operation is enabled 
     */
    pSetup->periCntlSetup.loopback = 0;
    
    /* Boot process is over (complete) */
    pSetup->periCntlSetup.bootComplete = bootcomplete;
    
    /* Process TX requests of priority 2, when credit is 1 or greater */
    pSetup->periCntlSetup.txPriority2Wm = CSL_SRIO_TX_PRIORITY_WM_1;
    
    /* Process TX requests of priority 1, when credit is 1 or greater */
    pSetup->periCntlSetup.txPriority1Wm = CSL_SRIO_TX_PRIORITY_WM_2;

    /* Process TX requests of priority 0, when credit is 1 or greater */
    pSetup->periCntlSetup.txPriority0Wm = CSL_SRIO_TX_PRIORITY_WM_3;

    /* Set internal bus priority to 4 */
    pSetup->periCntlSetup.busTransPriority = CSL_SRIO_BUS_TRANS_PRIORITY_4;
    
    /* UDI buffers are port based not proirity based */
    pSetup->periCntlSetup.bufferMode = CSL_SRIO_1X_MODE_PORT;
    
    /* VBUS clock is of 333 MHz  from TPCC document section 2.3 */
    pSetup->periCntlSetup.prescalar = CSL_SRIO_CLK_PRESCALE_1;
    
    /* Enable port 0 PLL */
    pSetup->periCntlSetup.pllEn = 0/*CSL_SRIO_PLL1_ENABLE*/; 

    /* Enable clocks to all domains */
    pSetup->gblEn = 1;

    /* Enable clock in all the domains */
    for (index=0; index<9; index++)
        pSetup->blkEn[index] = 1;

    /* 8-bit id is 0xAC and 16-bit id is 0xFACE */
    pSetup->deviceId1 = CSL_FMK(SRIO_BASE_ID_BASE_DEVICEID, SMALL_DEV_ID) | 
                        CSL_FMK(SRIO_BASE_ID_LARGE_BASE_DEVICEID, LARGE_DEV_ID);
    
    /* 8-bit id is 0xAC and 16-bit id is 0xFACE for multi-cast*/
    pSetup->deviceId2 = CSL_FMK(SRIO_BASE_ID_BASE_DEVICEID, SMALL_DEV_ID) | 
                        CSL_FMK(SRIO_BASE_ID_LARGE_BASE_DEVICEID, LARGE_DEV_ID);

	/* SERDES PLL configuration for channel 0 */
	pSetup->serDesPllCfg.pllEnable      = TRUE;
	pSetup->serDesPllCfg.pllMplyFactor  = CSL_SRIO_SERDES_PLL_MPLY_BY_10;
	pSetup->serDesPllCfg.loopBandwidth  = CSL_SRIO_SERDES_LOOP_BANDWIDTH_FREQ_DEP;

    /* SERDES RX channel 0 enable */
    pSetup->serDesRxChannelCfg[0].enRx              = TRUE; 
    pSetup->serDesRxChannelCfg[0].symAlign          = CSL_SRIO_SERDES_SYM_ALIGN_COMMA; 
	pSetup->serDesRxChannelCfg[0].los               = CSL_SRIO_SERDES_LOS_DET_DISABLE;
	pSetup->serDesRxChannelCfg[0].clockDataRecovery = 0x00;  /* first order */
	pSetup->serDesRxChannelCfg[0].equalizer         = 0x01;
	pSetup->serDesRxChannelCfg[0].termination       = CSL_SRIO_SERDES_TERMINATION_0_8_VDDT;
    pSetup->serDesRxChannelCfg[0].busWidth          = CSL_SRIO_SERDES_BUS_WIDTH_10_BIT;
    pSetup->serDesRxChannelCfg[0].rate              = CSL_SRIO_SERDES_RATE_HALF;

    /* SERDES TX channel 0 enable */
    pSetup->serDesTxChannelCfg[0].enTx              = TRUE;
    pSetup->serDesTxChannelCfg[0].commonMode        = CSL_SRIO_SERDES_COMMON_MODE_RAISED;
    pSetup->serDesTxChannelCfg[0].outputSwing       = CSL_SRIO_SERDES_SWING_AMPLITUDE_750;
    pSetup->serDesTxChannelCfg[0].rate              = CSL_SRIO_SERDES_RATE_HALF; 
	pSetup->serDesTxChannelCfg[0].enableFixedPhase  = FALSE; 
    
    /* SERDES RX channel 1 enable */
    pSetup->serDesRxChannelCfg[1].enRx              = TRUE; 
    pSetup->serDesRxChannelCfg[1].symAlign          = CSL_SRIO_SERDES_SYM_ALIGN_COMMA; 
	pSetup->serDesRxChannelCfg[1].los               = CSL_SRIO_SERDES_LOS_DET_DISABLE;
	pSetup->serDesRxChannelCfg[1].clockDataRecovery = 0x00;  /* first order */
	pSetup->serDesRxChannelCfg[1].equalizer         = 0x01;
	pSetup->serDesRxChannelCfg[1].termination       = CSL_SRIO_SERDES_TERMINATION_0_8_VDDT;
    pSetup->serDesRxChannelCfg[1].busWidth          = CSL_SRIO_SERDES_BUS_WIDTH_10_BIT;
    pSetup->serDesRxChannelCfg[1].rate              = CSL_SRIO_SERDES_RATE_HALF;

    /* SERDES TX channel 1 enable */
    pSetup->serDesTxChannelCfg[1].enTx              = TRUE; 
    pSetup->serDesTxChannelCfg[1].commonMode        = CSL_SRIO_SERDES_COMMON_MODE_RAISED;
    pSetup->serDesTxChannelCfg[1].outputSwing       = CSL_SRIO_SERDES_SWING_AMPLITUDE_750;
    pSetup->serDesTxChannelCfg[1].rate              = CSL_SRIO_SERDES_RATE_HALF; 
	pSetup->serDesTxChannelCfg[1].enableFixedPhase  = FALSE; 

    /* Select flow control ID length 16-bit */
    pSetup->flowCntlIdLen[0] = 1; 

    /* Destination ID of flow n, same ids as we are doing loopback */
    pSetup->flowCntlId[0] = LARGE_DEV_ID; 

    /* Sets the number of address bits generated by the PE as a source and 
     * processed by the PE as the target of an operation as 34 bits */
    pSetup->peLlAddrCtrl = CSL_SRIO_ADDR_SELECT_34BIT;

    /* Base device configuration */
    pSetup->devIdSetup.smallTrBaseDevId  = SMALL_DEV_ID;
    pSetup->devIdSetup.largeTrBaseDevId  = LARGE_DEV_ID;
    pSetup->devIdSetup.hostBaseDevId     = LARGE_DEV_ID;

    /* Port General configuration */
    pSetup->portGenSetup.portLinkTimeout = 0xFFFFF;  /* 215 ms */
    pSetup->portGenSetup.portRespTimeout = 0xFFFFF;  /* 215 ms */
    pSetup->portGenSetup.hostEn          = 1;        /* Agent/Slave Device           */
    pSetup->portGenSetup.masterEn        = 1;        /* Device cannot issue requests */

    /* Port control configuration for PORT 0 */
    pSetup->portCntlSetup[0].portDis           = 0;         /* Do not disable Port 0 */
    pSetup->portCntlSetup[0].outPortEn         = 1;         /* Output on Port 0 enabled */
    pSetup->portCntlSetup[0].inPortEn          = 1;         /* Input on Port 0 enabled */
    pSetup->portCntlSetup[0].portWidthOverride = CSL_SRIO_PORT_WIDTH_NO_OVERRIDE;
    pSetup->portCntlSetup[0].errCheckDis       = 0;         /* Err check enabled */
    pSetup->portCntlSetup[0].multicastRcvEn    = 0;         /* Multicast receive Disabled */
    pSetup->portCntlSetup[0].stopOnPortFailEn  = 0;         /* Dont Stop on fail */
    pSetup->portCntlSetup[0].dropPktEn         = 0;         /* Dont Drop PKT */
    pSetup->portCntlSetup[0].portLockoutEn     = 0;         /* Send any PKT */

    /* Port control configuration for PORT 1 */
    pSetup->portCntlSetup[1].portDis           = 0;        /* Do not disable Port 1 */
    pSetup->portCntlSetup[1].outPortEn         = 1;        /* Output on Port 1 enabled */
    pSetup->portCntlSetup[1].inPortEn          = 1;        /* Input on Port 1 enabled */
    pSetup->portCntlSetup[1].portWidthOverride = CSL_SRIO_PORT_WIDTH_NO_OVERRIDE;
    pSetup->portCntlSetup[1].errCheckDis       = 0;        /* Err check enabled */
    pSetup->portCntlSetup[1].multicastRcvEn    = 0;        /* Multicast receive Disabled. */
    pSetup->portCntlSetup[1].stopOnPortFailEn  = 0;        /* Dont Stop on fail */
    pSetup->portCntlSetup[1].dropPktEn         = 0;        /* Dont Drop PKT */
    pSetup->portCntlSetup[1].portLockoutEn     = 0;        /* Send any PKT */

    /* Enable all logical/transport errors */
    pSetup->lgclTransErrEn = CSL_SRIO_IO_ERR_RESP_ENABLE            |
                             CSL_SRIO_ILL_TRANS_DECODE_ENABLE       |
                             CSL_SRIO_ILL_TRANS_TARGET_ERR_ENABLE   |
                             CSL_SRIO_PKT_RESP_TIMEOUT_ENABLE       |
                             CSL_SRIO_UNSOLICITED_RESP_ENABLE       |
                             CSL_SRIO_UNSUPPORTED_TRANS_ENABLE;

    /* Enable all Port errors */    
    pSetup->portErrSetup[1].portErrRateEn =
                            CSL_SRIO_ERR_IMP_SPECIFIC_ENABLE            |
                            CSL_SRIO_CORRUPT_CNTL_SYM_ENABLE            |
                            CSL_SRIO_CNTL_SYM_UNEXPECTED_ACKID_ENABLE   |
                            CSL_SRIO_RCVD_PKT_NOT_ACCPT_ENABLE          |
                            CSL_SRIO_PKT_UNEXPECTED_ACKID_ENABLE        |
                            CSL_SRIO_RCVD_PKT_WITH_BAD_CRC_ENABLE       |
                            CSL_SRIO_RCVD_PKT_OVER_276B_ENABLE          |
                            CSL_SRIO_NON_OUTSTANDING_ACKID_ENABLE       |
                            CSL_SRIO_PROTOCOL_ERROR_ENABLE              |
                            CSL_SRIO_UNSOLICITED_ACK_CNTL_SYM_ENABLE    |
                            CSL_SRIO_LINK_TIMEOUT_ENABLE;

    /* Decrement error rate counter every second */ 
    pSetup->portErrSetup[1].prtErrRtBias = CSL_SRIO_ERR_RATE_BIAS_1S;

    /* Allow only 2 errors after error threshold is reached */ 
    pSetup->portErrSetup[1].portErrRtRec = CSL_SRIO_ERR_RATE_COUNT_2;

    /*  Port error setup */ 
    pSetup->portErrSetup[1].portErrRtFldThresh   = 10; /* Err threshold = 10 */
    pSetup->portErrSetup[1].portErrRtDegrdThresh = 10; /* Err degrade threshold = 10 */
    
    /* Configure the Port IP Mode (SP_IP_MODE) Register. 
     *  IDLE_ERR_DIS    - 0b0;  IDLE Error checking enabled
     *  TX_FIFO_BYP ASS - 0b0;  The TX_FIFO is operational
     *  PW_DIS          - 0b0;  Port-Write Error reporting is enabled
     *  TGT_ID_DIS      - 0b0;  packet accepted if DestID != BaseID
     *  SELF_RST        - 0b0;  Self reset disabled
     *  MLTC_EN         - 0b0;  Multicast-Event Interrupt Disable 
     *  RST_EN          - 0b0;  Reset Interrupt Disable 
     *  PW_EN           - 0b0;  Port-Write-In Interrupt Disable 
    */
    pSetup->portIpModeSet = 0x4D000000;

    /* Configure the SP_IP_PRESCALE register */
    pSetup->portIpPrescalar = 8; 

    /*  Port-Write Timer. The timer defines a period to repeat sending an error
     *  reporting Port-Write request for software assistance. The timer stopped 
     *  by software writing to the error detect registers 900 ms
     */
    pSetup->pwTimer = CSL_SRIO_PW_TIME_8;

    /* Port control independent error reporting enable. Macros can be ORed
     *  to get the value
     */

    /*
       TX_FLW        - 0;     Receive link flow control
       SOFT_REC      - 0;     Hardware controlled error recovery 
       FORCE_REINIT  - 0;     Reinitialization process NOT forced
       TRANS_MODE    - 01;    transfer mode - Store & Forward Mode 
       DEBUG         - 1;     Debug enable
       SEND_DBG_PKT  - 0;     Do not force a debug packet
       ILL_TRANS_EN  - 1;     Illegal Transfer Error reporting Enable
       MAX_RETRY_EN  - 1;     Max_retry_error report enable
       MAX_RETRY_THR - 0x01;  Maximum Retry Threshold Trigger
       IRQ_EN        - 1;     Interrupt error report Enable
    */ 
    pSetup->portCntlIndpEn[1] = 0x01A20180; 
}

/*
 * ============================================================================
 *   @func   main
 *
 *   @desc
 *     This is the main routine of this program which invokes the different
 *     components to complete a simple LSU transfer.
 * ============================================================================
*/
void main (void)
{
    Uint32              index;
    CSL_SrioContext     context;
    CSL_Status          status;
    CSL_SrioHandle      hSrio;
    CSL_SrioObj         srioObj;
    CSL_SrioParam       srioParam;
    CSL_SrioPortData    response;
    volatile Uint8*     shareBuf=DST;
    Uint32              counter = 1;

    /* Enable SRIO in PSC control module */
    enableSRIO();
   
    /* Initialization and Open of the SRIO */
    status = CSL_srioInit (&context);
    hSrio = CSL_srioOpen (&srioObj, 0, &srioParam, &status);
    if (status != CSL_SOK) {
        printf("SRIO: ... Cannot open SRIO, failed\n");
        return;
    }

    /* Disable the SRIO Peripheral and all the blocks */
    hSrio->regs->GBL_EN = 0;
    hSrio->regs->BLK_ENABLE[0].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[1].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[2].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[3].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[4].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[5].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[6].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[7].BLK_EN = 0x0;
    hSrio->regs->BLK_ENABLE[8].BLK_EN = 0x0;
    
    /* Disable PLL enable to disable the SerDes registers */
    hSrio->regs->SERDES_CFG_CNTL[0] = 0x0;
    hSrio->regs->SERDES_CFG_CNTL[1] = 0x0;
    hSrio->regs->SERDES_CFG_CNTL[2] = 0x0;
    hSrio->regs->SERDES_CFG_CNTL[3] = 0x0;

    /* Create the setup structure */
    srio_Create_Setup (&setup, 1);

	/* Setup the SRIO with the selected setup in the last step */
    status = CSL_srioHwSetup (hSrio, &setup);
    if (status != CSL_SOK) {
       	printf("SRIO: ... Hardware setup, failed\n");
        return;
    }

    /* Make sure that the SRIO Port is Initialized. */
    printf ("Debug: Waiting for SRIO Port %d to be initialized\n", SRIO_PORT_NUMBER);
    
    /* Loop around till the PORT is initialized. */
    response.index = SRIO_PORT_NUMBER;
    do 
    {
 		delay(10000);  //check the response every 100 cpu clcok = 100 ns
		CSL_srioGetHwStatus (hSrio, CSL_SRIO_QUERY_SP_ERR_STAT, &response);
    }while(response.data & 0x1);

    /* Loop around forever waiting for data to arrive. */
    while (1)
    {
        /* Setup the Destination */
        shareBuf = DST;
        for(index=0; index<TRANSFER_SIZE; index++)
	        shareBuf[index] = 0;

        /* Read the Port Error Status register. */
        response.index = SRIO_PORT_NUMBER;
        CSL_srioGetHwStatus (hSrio, CSL_SRIO_QUERY_SP_ERR_STAT, &response);

        /* Was there an error detected? */
        if ((response.data & ~0x2) != 0)
        {
            /* Error was detected; we need to clear this before we can proceed... */
            CSL_srioHwControl (hSrio, CSL_SRIO_CMD_SP_ERR_STAT_CLEAR, &response);
        }

        /* Wait for the transfer to be completed. */
        printf ("Debug: Waiting for the transfer %d...\n", counter);
        while(shareBuf[TRANSFER_SIZE - 1] == 0);

        /* Verify whether data is transfered */
        for(index=0; index<TRANSFER_SIZE; index++) 
        {
	        if (shareBuf[index] != index)
		        break;
        }

        if (index == TRANSFER_SIZE)
	        printf("Debug: Transfer %d was good.\n", counter);
        else
	        printf("Error: Transfer %d was corrupt.\n", counter);

        /* Increment the transfer counter. */
        counter++;        
    }
}

