/* =============================================================================
 * Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006
 *
 * Use of this software is controlled by the terms and conditions found in the
 * license agreement under which this software has been supplied.
 * ==========================================================================*/

/** ===========================================================================
 * @file Timer_example.c
 *
 * @path $(CSLPATH)\example\timer\src
 *
 * @desc Example for the usage of General purpose timer CSL APIs
 * =============================================================================
 * @n <b> Test Description </b>
 * @n The example illustrates the usage of CSL APIs for using timer.
 * 1. This example configures the first instance of timer
 *    as interrupt or event to be sourced normally by the timer.
 * 2. This enables the timer in one shot mode (for watchdog mode timer is
 *    enabled in continuous mode).
 * 3. The  interrupt handler announces occurrence of each interrupt
 *    to the application.
 * 4. The above said procedure is followed for all the modes of the timer
 *    (i.e. 64 bit timer, dual 32 bit timer( chained and unchained)
 *    and watchdog).
 **************************************************************************
 */

/** ============================================================================
 *  Revision History
 *  ===============
 * 29-Jul-2005 PSK updated changes according to revised timer spec. the number
 *                 convention TIM12, TIM34 are changed to TIMLO and TIMHI.
 * 21-Apr-2005 PSK File Created.
 * 12-Jan-2009 Removed Watchdog timer test. Moved to separate example.
 * =============================================================================
 */

#include <csl_tmr.h>
#include <stdio.h>
#include <csl_intc.h>
#include <soc.h>
#include <csl_intcAux.h>
#include <csl_tmrAux.h>

/** Handle for the TMR instance */
CSL_TmrHandle         hTmr;

/** change the Instance of the timer and test */
Uint8 IntcInstance = CSL_TMR_0;


/* Locals & Forwards */
void tmrIntcPrdLoLoadDemo(void);
void tmrIntcPrdHiLoadDemo(void);
void tmrIntcChainedDemo(void);
void tmrIntcGptDemo(void);

/*
 *****************************************************************************
 *                    INTC related code
 *****************************************************************************
 */

/* Intc Declarations */
CSL_IntcObj                  tmrIntcObj, tmrIntcObj1;
CSL_IntcHandle               tmrIntcHandle, tmrIntcHandle1;
CSL_IntcContext              context, context1;

CSL_IntcEventHandlerRecord   eventHandler[30];
CSL_IntcEventHandlerRecord   record[1];

/* count for ISR */
volatile int intrCnt = 0;

/*
 * ===========================================================================
 * @func   TimerInterruptHandler
 *
 * @desc
 *    Handler for Timer Interrupt
 *
 * @arg    event - interrupt event id
 * ============================================================================
 */
void TimerInterruptHandler (
    CSL_IntcEventId * event
)
{
    intrCnt++;
    CSL_intcEventClear(*event);
}



/** ===========================================================================
 *  @n@b   main
 *
 *  @b Description
 *  @n This is the main function for the file.This initializes the CSL for
 *     INTC, installs interrupt handler for first instance of general purpose
 *     timer and invokes the routine which demonstrates the usage of CSL APis
 *     for General purpose timer
 *
 *  @b Arguments
 *  @verbatim
            None
    @endverbatim
 *
 * <b> Return Value </b>  void
 * ============================================================================
 */
void main (
    void
)
{
    CSL_IntcGlobalEnableState state;

    /* Initialize INTC */
    context.numEvtEntries = 1;
    context.eventhandlerRecord = record;

    CSL_intcInit(&context);

    /*Enable NMIS */
    CSL_intcGlobalNmiEnable();

    /* Enable all interrupts */
    CSL_intcGlobalEnable(&state);

    tmrIntcPrdLoLoadDemo();
    tmrIntcPrdHiLoadDemo();
    tmrIntcChainedDemo();
    tmrIntcGptDemo();

    printf(">>>>>> Examples for all the modes of timer are "
    		"completed and PASSED <<<<<<<< \n");
}

/*
 * ===========================================================================
 * @func   tmrIntcPrdLoLoadDemo
 *
 * @desc
 *    This function is example for the Period register loading with value
 *
 * @arg    None
 * ============================================================================
 */
void tmrIntcPrdLoLoadDemo (
    void
)
{
    CSL_TmrObj TmrObj;
    CSL_Status intStat, status;
    CSL_TmrHwSetup hwSetup = CSL_TMR_HWSETUP_DEFAULTS;
    CSL_IntcEventHandlerRecord EventRecord;
    CSL_IntcParam vectId;

    Uint32 LoadValue = 100;

    CSL_TmrEnamode TimeCountMode = CSL_TMR_ENAMODE_CONT;

    /* Clear local data structures */
    memset(&TmrObj, 0, sizeof(CSL_TmrObj));
    printf("Running Gp Timer Interrupt DUAL UNCHAINED Low....\n");

    /**************************************************************
    * INTC related code      						  *
    **************************************************************/

    /* Open INTC */
    vectId = CSL_INTC_VECTID_12;

    tmrIntcHandle = CSL_intcOpen(&tmrIntcObj, CSL_INTC_EVENTID_TINTLO0, &vectId,
                                 &intStat);

    /* Bind ISR to Interrupt */
    EventRecord.handler = (CSL_IntcEventHandler)&TimerInterruptHandler;
    EventRecord.arg = (void *)CSL_INTC_EVENTID_TINTLO0;
    CSL_intcPlugEventHandler(tmrIntcHandle, &EventRecord);

    /* Event Enable */
    CSL_intcHwControl(tmrIntcHandle, CSL_INTC_CMD_EVTENABLE, NULL);

    /**************************************************************/

    /* Initialize GP timer CSL module */
    CSL_tmrInit(NULL);

    hTmr =  CSL_tmrOpen(&TmrObj, IntcInstance, NULL, &status);

    /* Stop the GP Timer */
    CSL_TmrResetLo(hTmr);

    /* set the timer mode to unchained dual mode */
    hwSetup.tmrTimerMode = CSL_TMR_TIMMODE_DUAL_UNCHAINED;
    hwSetup.tmrTimerPeriodLo = LoadValue;
    CSL_tmrHwSetup(hTmr, &hwSetup);

    /* Start the timer in continuous mode */
    CSL_TmrStartLo(hTmr, TimeCountMode);

    /* INTC related code: Wait for multiple events to occur. */
    while (1) {
        if (intrCnt == 5)
	        break;
    }

    /**************************************************************/

    CSL_intcHwControl(tmrIntcHandle, CSL_INTC_CMD_EVTDISABLE, NULL);

    printf("INTR: The Total number of Events occurred are: 0x%d\n", intrCnt);

    intrCnt = 0;

    /* Stop the GP Timer */
    CSL_TmrResetLo(hTmr);

    printf("Example for TIMLO completed\n");

    /* Close the Tmr instance */
    CSL_tmrClose(hTmr);
    CSL_intcClose(tmrIntcHandle);
}

/*
 * =============================================================================
 *   @func   tmrIntcPrdHiLoadDemo
 *
 *   @desc
 *      This function tests the Period register High loading with value
 *
 *   @arg    None
 * =============================================================================
 */
void tmrIntcPrdHiLoadDemo (
    void
)
{
    CSL_TmrObj TmrObj;
    CSL_Status status;
    CSL_TmrHwSetup hwSetup = CSL_TMR_HWSETUP_DEFAULTS;
    CSL_IntcEventHandlerRecord EventRecord;
    CSL_IntcParam vectId;

    Uint32 LoadValue = 100;

    CSL_TmrEnamode TimeCountMode = CSL_TMR_ENAMODE_ENABLE;

    /* Clear local data structures */
    memset(&TmrObj, 0, sizeof(CSL_TmrObj));
    printf("\n\n\n\n\nRunning Gp Timer Interrupt DUAL UNCHAINED TIMHI....\n");

    /**************************************************************
    * INTC related code 						  *
    **************************************************************/

    /* Open INTC	*/
    vectId = CSL_INTC_VECTID_13;
    tmrIntcHandle = CSL_intcOpen(&tmrIntcObj, CSL_INTC_EVENTID_TINTHI0, &vectId,
                                 NULL);

    /* Bind ISR to Interrupt */
    EventRecord.handler = (CSL_IntcEventHandler)&TimerInterruptHandler;
    EventRecord.arg = (void *)CSL_INTC_EVENTID_TINTHI0;
    CSL_intcPlugEventHandler(tmrIntcHandle, &EventRecord);

    /* Event Enable */
    CSL_intcHwControl(tmrIntcHandle, CSL_INTC_CMD_EVTENABLE, NULL);

    /**************************************************************/

    /* Initialize GP timer CSL module */
    CSL_tmrInit(NULL);
    hTmr =  CSL_tmrOpen(&TmrObj, IntcInstance, NULL, &status);

    /* Stop the GP Timer */
    CSL_TmrResetHi(hTmr);

    /* set the timer mode to unchained dual mode */
    hwSetup.tmrTimerMode = CSL_TMR_TIMMODE_DUAL_UNCHAINED;
    hwSetup.tmrTimerPeriodHi = LoadValue;
    CSL_tmrHwSetup(hTmr, &hwSetup);

    /* Start the timer in one shot mode*/
    CSL_TmrStartHi(hTmr, TimeCountMode);

    /* INTC related code */
    while (1) {
        if (intrCnt == 1)
	        break;
    }

    /**************************************************************/

    CSL_intcHwControl(tmrIntcHandle, CSL_INTC_CMD_EVTDISABLE, NULL);

    printf("INTR: The Total number of Events occurred are: 0x%d\n", intrCnt);

    intrCnt = 0;

    /* Stop the GP Timer */
    CSL_TmrResetHi(hTmr);
    printf("Example for TIM34 completed\n");

    /* Close the Tmr instance */
    CSL_tmrClose(hTmr);
    CSL_intcClose(tmrIntcHandle);
}

/*
 * ============================================================================
 *   @func   tmrIntcChainedDemo
 *
 *   @desc
 *     This test verifies the Setting of timer to dual 32 bit chained
 *     and one shot mode with INTC
 *
 *   @arg    None
 * ============================================================================
*/
void tmrIntcChainedDemo (
    void
)
{
    CSL_TmrObj TmrObj;
    CSL_Status status;
    CSL_IntcEventHandlerRecord EventRecord;
    CSL_IntcParam vectId;

    Uint32 LoadValue = 100;
    Uint32 LoadValue1 = 0;
    Uint8 preScalValue = 0;

    CSL_TmrHwSetup hwSetup = CSL_TMR_HWSETUP_DEFAULTS;

    /* set the timer in to one shot mode */
    CSL_TmrEnamode TimeCountMode = CSL_TMR_ENAMODE_ENABLE;

    /* Clear local data structures */
    memset(&TmrObj, 0, sizeof(CSL_TmrObj));
    printf("\n\n\n\n\nRunning Gp Timer Interrupt DUAL CHAINED....\n");

    /**************************************************************
    * INTC related code       						  *
    **************************************************************/

    /* Open INTC */
    vectId = CSL_INTC_VECTID_12;
    tmrIntcHandle = CSL_intcOpen(&tmrIntcObj, CSL_INTC_EVENTID_TINTLO0, \
                                                  &vectId, NULL);

    /* Bind ISR to Interrupt */
    EventRecord.handler = (CSL_IntcEventHandler)&TimerInterruptHandler;
    EventRecord.arg = (void *)CSL_INTC_EVENTID_TINTLO0;
    CSL_intcPlugEventHandler(tmrIntcHandle, &EventRecord);

    /* Event Enable */
    CSL_intcHwControl(tmrIntcHandle, CSL_INTC_CMD_EVTENABLE, NULL);

    /**************************************************************/

    /* Initialize GP timer CSL module */
    CSL_tmrInit(NULL);

    hTmr =  CSL_tmrOpen(&TmrObj, IntcInstance, NULL, &status);

    /* Reset the GP Timer */
    CSL_TmrReset64(hTmr);

    /* set the timer mode to dual chained mode */
    hwSetup.tmrTimerMode = CSL_TMR_TIMMODE_DUAL_CHAINED;
    hwSetup.tmrPreScalarCounterHi = preScalValue;
    hwSetup.tmrTimerPeriodLo = LoadValue;
    hwSetup.tmrTimerPeriodHi = LoadValue1;
    CSL_tmrHwSetup(hTmr, &hwSetup);

    /* Start the timer with one shot*/
    CSL_TmrStart64(hTmr, TimeCountMode);

	/**************************************************************
    * INTC related code 						  *
    **************************************************************/
    while (1) {
        if (intrCnt == 1)
	        break;
	}
    /**************************************************************/


    CSL_intcHwControl(tmrIntcHandle, CSL_INTC_CMD_EVTDISABLE, NULL);

    printf("INTR: The Total number of Events occurred are: 0x%d\n", intrCnt);

    intrCnt = 0;

    /* Stop the GP Timer */
    CSL_TmrReset64(hTmr);

    printf("Example for timer chained mode\n");

    /* Close the Tmr instance */
    CSL_tmrClose(hTmr);
    CSL_intcClose(tmrIntcHandle);
}

/*
 * =============================================================================
 *   @func   tmrIntcGptDemo
 *
 *   @desc
 *     This test verifies the Setting of timer to GPT mode with INTC
 *
 *   @arg    None
 * =============================================================================
*/
void tmrIntcGptDemo (
    void
)
{
    CSL_TmrObj TmrObj;
    CSL_IntcEventHandlerRecord EventRecord, EventRecord1;
    CSL_IntcParam vectId;
    CSL_Status status;

    CSL_TmrHwSetup hwSetup = CSL_TMR_HWSETUP_DEFAULTS;

    /* set the timer in one shot mode */
    CSL_TmrEnamode TimeCountMode = CSL_TMR_ENAMODE_ENABLE;

    /* set the timer mode to GPT mode */
    hwSetup.tmrTimerMode = CSL_TMR_TIMMODE_GPT;

    /* load the PRDLO */
    hwSetup.tmrTimerPeriodLo = 0x0f;

    /* load the PRDHI */
    hwSetup.tmrTimerPeriodHi = 0x00;

    /* Clear local data structures */
    memset(&TmrObj, 0, sizeof(CSL_TmrObj));
    printf("\n\n\n\nRunning Gp Timer with INTC....\n");

    /**************************************************************
    * INTC related code            					  *
    **************************************************************/

    /* Open INTC */
    vectId = CSL_INTC_VECTID_12;
    tmrIntcHandle = CSL_intcOpen(&tmrIntcObj, CSL_INTC_EVENTID_TINTLO0,\
                                                &vectId, NULL);

    vectId = CSL_INTC_VECTID_13;
    tmrIntcHandle1 = CSL_intcOpen(&tmrIntcObj1, CSL_INTC_EVENTID_TINTHI0,\
                                                &vectId, NULL);

    /* Bind ISR to Interrupt */
    EventRecord.handler = (CSL_IntcEventHandler)&TimerInterruptHandler;
    EventRecord.arg = (void *)CSL_INTC_EVENTID_TINTLO0;
    CSL_intcPlugEventHandler(tmrIntcHandle, &EventRecord);

    EventRecord1.handler = (CSL_IntcEventHandler)&TimerInterruptHandler;
    EventRecord.arg = (void *)CSL_INTC_EVENTID_TINTHI0;
    CSL_intcPlugEventHandler(tmrIntcHandle1, &EventRecord1);

    /* Event Enable */
    CSL_intcHwControl(tmrIntcHandle, CSL_INTC_CMD_EVTENABLE, NULL);
    CSL_intcHwControl(tmrIntcHandle1, CSL_INTC_CMD_EVTENABLE, NULL);

    /**************************************************************/

    /* Initialize GP timer CSL module */
    CSL_tmrInit(NULL);

    hTmr = CSL_tmrOpen(&TmrObj, IntcInstance, NULL, &status);

    CSL_tmrHwSetup(hTmr, &hwSetup);

    /* Reset the GP Timer */
    CSL_TmrReset64(hTmr);

    /* Start the timer with one shot*/
    CSL_TmrStart64(hTmr, TimeCountMode);

	/**************************************************************
    * INTC related code 						  *
    **************************************************************/
    while (1) {
        if (intrCnt == 1)
	        break;
    }
    /**************************************************************/

    CSL_intcHwControl(tmrIntcHandle, CSL_INTC_CMD_EVTDISABLE, NULL);

    CSL_intcHwControl(tmrIntcHandle1, CSL_INTC_CMD_EVTDISABLE, NULL);

    printf("INTR: The Total number of Events occurred are: 0x%d\n", intrCnt);

    intrCnt = 0;

    /* Stop the GP Timer */
    CSL_TmrReset64(hTmr);

    printf("Example for 64-bit timer mode\n");

    /* Close the Tmr instance */
    CSL_tmrClose(hTmr);
    CSL_intcClose(tmrIntcHandle);
}




