/* =============================================================================
 * Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006
 *
 * Use of this software is controlled by the terms and conditions found in the
 * license agreement under which this software has been supplied.
 * ==========================================================================*/

/** ===========================================================================
 * @file Watchdog_example.c
 *
 * @path $(CSLPATH)\example\timer\src
 *
 * @desc Example for the usage of General purpose timer CSL APIs
 * =============================================================================
 * @n <b> Test Description </b>
 * @n The example illustrates the usage of CSL APIs for using timer
 *    in watchdog mode.
 * 1. This example configures the first instance of timer
 *    as interrupt or event to be sourced normally by the timer.
 * 2. For watchdog mode timer is enabled in continuous mode.
 * 3. The  interrupt handler announces occurrence of each interrupt
 *    to the application.
 * 4. Once this example is executed the target must be hard reset 
 *    or power cycled to reset the Watchdog mode..
 * 5. The test case configures the timer to operate in Watchdog mode so please
 *    ensure that the hardware is reset before executing any of the timer 
 *    examples.
 **************************************************************************
 */

/** ============================================================================
 *  Revision History
 *  ===============
 * 12-Jan-2009 Created separate example for Watchdog timer.
 * =============================================================================
 */

#include <csl_tmr.h>
#include <stdio.h>
#include <csl_intc.h>
#include <soc.h>
#include <csl_intcAux.h>
#include <csl_tmrAux.h>

/** Handle for the TMR instance */
CSL_TmrHandle         hTmr;

/** change the Instance of the timer and test */
Uint8 IntcInstance = CSL_TMR_0;


void tmrWdtModeDemo(void);

/*
 *****************************************************************************
 *                    INTC related code
 *****************************************************************************
 */

/* Intc Declarations */
CSL_IntcObj                  tmrIntcObj;
CSL_IntcHandle               tmrIntcHandle;
CSL_IntcContext              context;

//CSL_IntcEventHandlerRecord   eventHandler[30];
CSL_IntcEventHandlerRecord   record[1];

/* count for ISR */
volatile int intrCnt = 0;

/*
 * ===========================================================================
 * @func   TimerInterruptHandler
 *
 * @desc
 *    Handler for Timer Interrupt
 *
 * @arg    event - interrupt event id
 * ============================================================================
 */
void TimerInterruptHandler (
    CSL_IntcEventId * event
)
{
    intrCnt++;
    CSL_intcEventClear(*event);
}

/** ===========================================================================
 *  @n@b   main
 *
 *  @b Description
 *  @n This is the main function for the file.This initializes the CSL for
 *     INTC, installs interrupt handler for first instance of general purpose
 *     timer and invokes the routine which demonstrates the usage of CSL APis
 *     for General purpose timer
 *
 *  @b Arguments
 *  @verbatim
            None
    @endverbatim
 *
 * <b> Return Value </b>  void
 * ============================================================================
 */
void main (
    void
)
{
    CSL_IntcGlobalEnableState state;

    /* Initialize INTC */
    context.numEvtEntries = 1;
    context.eventhandlerRecord = record;

    CSL_intcInit(&context);

    /*Enable NMIS */
    CSL_intcGlobalNmiEnable();

    /* Enable all interrupts */
    CSL_intcGlobalEnable(&state);

    tmrWdtModeDemo();

    printf(">>>>>> Examples for Watchdog mode of timer is "
    		"completed and PASSED <<<<<<<< \n");
}


/*
 * ============================================================================
 *   @func   tmrWdtModeDemo
 *
 *   @desc
 *     This verifies the Setting of timer to watchdog mode with INTC
 *
 *   @arg    None
 * ============================================================================
 */
void tmrWdtModeDemo (
    void
)
{
    CSL_TmrObj TmrObj;
    CSL_Status status;
    CSL_IntcParam vectId;
    CSL_IntcEventHandlerRecord EventRecord;

    CSL_TmrHwSetup hwSetup = CSL_TMR_HWSETUP_DEFAULTS;

    /* set the timer in to continuous mode and it is must for watchdog mode */
    CSL_TmrEnamode TimeCountMode = CSL_TMR_ENAMODE_CONT;

    CSL_IntcEventId eventId = CSL_INTC_EVENTID_TINTLO0;

    Uint16 loadVal = CSL_TMR_WDTCR_WDKEY_CMD1;

    /* set the timer mode to WATCHDOG mode */
    hwSetup.tmrTimerMode = CSL_TMR_TIMMODE_WDT;
    hwSetup.tmrPulseWidthLo = CSL_TMR_PWID_THREECLKS;
    hwSetup.tmrClksrcLo = CSL_TMR_CLKSRC_INTERNAL;
    hwSetup.tmrClockPulseLo = CSL_TMR_CP_PULSE;
    hwSetup.tmrClockPulseHi = CSL_TMR_CP_PULSE;
    hwSetup.tmrIpGateLo = CSL_TMR_CLOCK_INP_NOGATE;

    /* load the PRDLO */
    hwSetup.tmrTimerPeriodLo = 0x100;

    /* load the PRDHI */
    hwSetup.tmrTimerPeriodHi = 0x0;

    /* Clear local data structures */
    memset(&TmrObj, 0, sizeof(CSL_TmrObj));
    printf("\n\n\n\n Running the GP timer in watchdog mode with INTC......\n");

    /**************************************************************
    * INTC related code 						  *
    **************************************************************/

    /* Open INTC */
    vectId = CSL_INTC_VECTID_12;

	if (IntcInstance == 1)
        eventId = eventId + 2;

    tmrIntcHandle = CSL_intcOpen(&tmrIntcObj, eventId, &vectId, NULL);

    /* Bind ISR to Interrupt */
    EventRecord.handler = (CSL_IntcEventHandler)&TimerInterruptHandler;
    EventRecord.arg = (void *)eventId;
    CSL_intcPlugEventHandler(tmrIntcHandle, &EventRecord);

    /* Event Enable */
    CSL_intcHwControl(tmrIntcHandle, CSL_INTC_CMD_EVTENABLE, NULL);

    /**************************************************************/

    /* Initialize GP timer CSL module */
    status = CSL_tmrInit(NULL);

    hTmr =  CSL_tmrOpen(&TmrObj, IntcInstance, NULL, &status);

    CSL_tmrHwSetup(hTmr, &hwSetup);

    /* Reset the GP Timer */
    status = CSL_tmrHwControl(hTmr, CSL_TMR_CMD_RESET64, NULL);

    /* Start the timer in continuous mode*/
    status = CSL_tmrHwControl(hTmr, CSL_TMR_CMD_START_WDT, \
                               (void *)&TimeCountMode);

    status = CSL_tmrHwControl(hTmr, CSL_TMR_CMD_LOAD_WDKEY, \
                               (Uint16 *) &loadVal);

    loadVal = CSL_TMR_WDTCR_WDKEY_CMD2;

    status = CSL_tmrHwControl(hTmr, CSL_TMR_CMD_LOAD_WDKEY, \
                               (Uint16 *)&loadVal);

    /**************************************************************
    * INTC related code and test								  *
    **************************************************************/
    while (1) {
        if (intrCnt == 1)
	        break;
    }

    /**************************************************************/

    CSL_intcHwControl(tmrIntcHandle, CSL_INTC_CMD_EVTDISABLE, NULL);

	printf("INTR: The Total number of Events occurred are: 0x%d\n", intrCnt);

    intrCnt = 0;

    /* Close the Tmr instance */
    status = CSL_tmrClose(hTmr);
    CSL_intcClose(tmrIntcHandle);

    printf("\nPLEASE DO HARDWARE or SYSTEM RESET TO RERUN THIS EXAMPLE BECAUSE OF WATCHDOG TIMEOUT \n");
    return;
}


