/* ============================================================================
 * Copyright (c) Texas Instruments Inc 2008
 *
 * Use of this software is controlled by the terms and conditions found in the
 * license agreement under which this software has been supplied.
 * ============================================================================
 */

/* =============================================================================
 * Revision History
 * ================
 *  19-Aug-2008     File Created
 * =============================================================================
 */

#include <csl_emac3x.h>

/** ============================================================================
 *   @n@b CSL_emacGetBaseAddress
 *
 *   @b Description
 *   @n  Function to get the base address of the peripheral instance.
 *       This function is used for getting the base address of the peripheral
 *       instance. This function will be called inside the CSL_emacOpen()
 *       function call. This function is open for re-implementing if the user
 *       wants to modify the base address of the peripheral object to point to
 *       a different location and there by allow CSL initiated write/reads into
 *       peripheral. MMR's go to an alternate location.
 *
 *   @b Arguments
 *   @verbatim
            emacNum         Specifies the instance of the EMAC to be opened.

            pEmacParam      Module specific parameters.

            pBaseAddress    Pointer to baseaddress structure containing base
                            address details.

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK            Successful on getting the base address
 *   @li                    CSL_ESYS_FAIL      The instance number is invalid.
 *   @li                    CSL_ESYS_INVPARAMS Inavlid parameters
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  Base Address structure is populated with EMAC and ECTL register's
 *       base addresses.
 *
 *   @b Modifies
 *   @n    1. The status variable
 *
 *         2. Base address structure is modified.
 *
 *   @b Example
 *   @verbatim
        CSL_Status              status;
        CSL_EmacBaseAddress     baseAddress;

       ...
      status = CSL_emacGetBaseAddress(CSL_EMAC, NULL, &baseAddress);

    @endverbatim
 * ===========================================================================
 */
#pragma CODE_SECTION (CSL_emacGetBaseAddress, ".text:csl_section:emac");
CSL_Status  CSL_emacGetBaseAddress (
    CSL_InstNum             emacNum,
    CSL_EmacParam           *pEmacParam,
    CSL_EmacBaseAddress     *pBaseAddress
)
{
    CSL_Status      status = CSL_SOK;

    /* Validate input */
    if (pBaseAddress == NULL)
        return CSL_ESYS_INVPARAMS;

    switch (emacNum)
    {
        /* Faraday has support for only one EMAC device */
        case CSL_EMAC:
            pBaseAddress->emacRegs  = (CSL_EmacRegsOvly)CSL_EMAC_0_REGS;
            pBaseAddress->ectlRegs  = (CSL_EctlRegsOvly)CSL_ECTL_0_REGS;
            break;

        /* Invalid EMAC peripheral number */
        default:
            pBaseAddress->emacRegs  = (CSL_EmacRegsOvly)NULL;
            pBaseAddress->ectlRegs  = (CSL_EctlRegsOvly)NULL;
            status = CSL_ESYS_FAIL;
            break;
    }

    return status;
}
