/*  ============================================================================
 *  Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006
 *
 *  Use of this software is controlled by the terms and conditions found in the
 *  license agreement under which this software has been supplied.
 *  ============================================================================
 */

/** @file csl_pscGetBaseAddress.c
 *
 *  @brief File for functional layer of CSL API @a CSL_pscGetBaseAddress()
 *
 *  Description
 *    - The @a CSL_pscGetBaseAddress() function definition & it's associated functions
 *
 *  Path: \\(CSLPATH)\\src\\common
 */

/* =============================================================================
 *  Revision History
 *  ===============
 *  29-Jun-2006 TRR File Created.
 * =============================================================================
 */

#include <soc.h>
#include <csl_psc.h>

/** ============================================================================
 *   @n@b CSL_pscGetBaseAddress
 *
 *   @b Description
 *   @n  This function gets the base address of the given PSC
 *       instance.
 *
 *   @b Arguments
 *   @verbatim
            pscNum        Specifies the instance of the PSC to be opened

            pTmrParam     Psc module specific parameters

            pBaseAddress  Pointer to base address structure containing base
                          address details
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK            Successful on getting the base 
 *                                             address
 *   @li					CSL_ESYS_FAIL      PSC instance is not
 *                                             available.
 *   @li                    CSL_ESYS_INVPARAMS Invalid Parameters
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  Base address structure is populated
 *
 *   @b Modifies
 *   @n    1. The status variable
 *
 *         2. Base address structure is modified.
 *
 *   @b Example
 *   @verbatim
        CSL_Status            status;
        CSL_PscBaseAddress    baseAddress;

        ...
        status = CSL_pscGetBaseAddress(CSL_PSC, NULL, &baseAddress);
        ...

    @endverbatim
 * ===========================================================================
 */
#pragma CODE_SECTION (CSL_pscGetBaseAddress, ".text:csl_section:psc");
CSL_Status CSL_pscGetBaseAddress (
    CSL_InstNum 	        pscNum,
    CSL_PscParam *          pPscParam,
    CSL_PscBaseAddress *    pBaseAddress
)
{
    CSL_Status st = CSL_SOK;

    pPscParam = pPscParam;

    if (pBaseAddress == NULL)
        return CSL_ESYS_INVPARAMS;

    switch (pscNum) {
    
    case CSL_PSC:
        pBaseAddress->regs = (CSL_PscRegsOvly)CSL_PSC_0_REGS;
        break;
    
    default:
        pBaseAddress->regs = (CSL_PscRegsOvly)NULL;
        st = CSL_ESYS_FAIL;
        break;
    }

    return st;
}

