/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
 *   ===========================================================================
 */

/** @file csl_pllcHwSetup.c
 *
 *  @brief    File for functional layer of CSL API @a CSL_pllcHwSetup()
 *
 *  Path: \\(CSLPATH)\src\pllc
 *
 *  Date 25 June, 2004
 *
 *  Author Pratheesh Gangadhar
 */

/* =============================================================================
 *  Revision History
 *  ===============
 *  10-Feb-2004 kpn CSL3X Upgradation.
 *  21-Apr-2006 RR   updated for faraday
 * =============================================================================
 */

#include <csl_pllc.h>

/** ============================================================================
 *   @n@b CSL_pllcHwSetup
 *
 *   @b Description
 *   @n It configures the pllc registers as per the values passed
 *      in the hardware setup structure.
 *
 *   @b Arguments
 *   @verbatim
            hPllc            Handle to the pllc

            hwSetup          Pointer to hardware setup structure
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Hardware setup successful
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle
 *   @li                    CSL_ESYS_INVPARAMS  - Hardware structure is not
                                                  properly initialized
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  PLLC registers are configured according
 *       to the hardware setup parameters
 *
 *   @b Modifies
 *   @n PLLC registers
 *
 *   @b Example
 *   @verbatim
            CSL_PllcHandle   hPllc;
            CSL_PllcObj      pllcObj;
            CSL_PllcHwSetup  hwSetup;
            CSL_status       status;
            ...

            hPllc = CSL_pllcOpen(&pllcObj, CSL_PLLC_0, NULL, &status);
            ...

            status = CSL_pllcHwSetup(hPllc, &hwSetup);
     @endverbatim
 * =============================================================================
 */
#pragma CODE_SECTION (CSL_pllcHwSetup, ".text:csl_section:pllc");
CSL_Status CSL_pllcHwSetup (
    CSL_PllcHandle            hPllc,
    CSL_PllcHwSetup          *hwSetup
)
{
    CSL_PllcRegsOvly pllcRegs = hPllc->regs;
    CSL_Status       status = CSL_SOK;
    volatile unsigned int loopCount;

    if (hPllc == NULL) {
        return CSL_ESYS_BADHANDLE;
    }

    if (hwSetup == NULL) {
        return CSL_ESYS_INVPARAMS;
    }
	
	//CSL_FINST(pllcRegs->PLLCTL, PLLC_PLLCTL_PLLENSRC, REGBIT);
    CSL_FINST(pllcRegs->PLLCTL, PLLC_PLLCTL_PLLEN, BYPASS);

    /* Wait for 4 cycles of the slowest of the  CLKIN/OSCIN
     * Pratheesh G: 06/29/2004
     * This part requires fine tuing, right now I am not aware of CLKIN/OSCIN
     * values. So introducing some arbitrary delay, if CPU speed is 27Mhz, loop
     * will take around 15 ms
     */

    loopCount = 100;
    while (loopCount--) {
        asm("   NOP");
    }

	CSL_FINST(pllcRegs->PLLCTL, PLLC_PLLCTL_PLLRST, YES);
	CSL_FINS(pllcRegs->PLLM, PLLC_PLLM_PLLM, hwSetup->pllM);
	pllcRegs->PREDIV = CSL_FMK(PLLC_PREDIV_RATIO, hwSetup->preDiv) |
	                   CSL_FMKT(PLLC_PREDIV_PREDEN, ENABLE);

	while (CSL_FEXT(pllcRegs->PLLSTAT, PLLC_PLLSTAT_GOSTAT)) {
        /* Poll for divide-ratio change and clock alignment to complete */
    }

    if (hwSetup->divEnable & CSL_PLLC_DIVEN_PLLDIV11) {
        CSL_FINST(pllcRegs->PLLDIV11, PLLC_PLLDIV11_D11EN, ENABLE);
        CSL_FINS(pllcRegs->PLLDIV11, PLLC_PLLDIV11_RATIO, hwSetup->pllDiv11);
    }
    else {
        CSL_FINST(pllcRegs->PLLDIV11, PLLC_PLLDIV11_D11EN, DISABLE);
    }

    if (hwSetup->divEnable & CSL_PLLC_DIVEN_PLLDIV13) {
        CSL_FINST(pllcRegs->PLLDIV13, PLLC_PLLDIV13_D13EN, ENABLE);
        CSL_FINS (pllcRegs->PLLDIV13, PLLC_PLLDIV13_RATIO, hwSetup->pllDiv13);
    }
    else {
        CSL_FINST(pllcRegs->PLLDIV13, PLLC_PLLDIV13_D13EN, DISABLE);
    }

    CSL_FINS (pllcRegs->PLLCMD, PLLC_PLLCMD_GOSET, CSL_PLLC_PLLCMD_GOSET_SET);

    while (CSL_FEXT (pllcRegs->PLLSTAT, PLLC_PLLSTAT_GOSTAT)) {
        /* Poll for divide-ratio change and clock alignment to complete */
    }

	loopCount = 1000;
    while (loopCount--) {
        asm("   NOP");
    }

    CSL_FINST(pllcRegs->PLLCTL, PLLC_PLLCTL_PLLRST, NO);

	loopCount = 4000;
    while (loopCount--) {
        asm("   NOP");
    }

    CSL_FINST(pllcRegs->PLLCTL, PLLC_PLLCTL_PLLEN, PLL);

    return status;
}

