/* ============================================================================
 * Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006
 *
 * Use of this software is controlled by the terms and conditions found in the
 * license agreement under which this software has been supplied.
 * ============================================================================
 */

/** ===========================================================================
 * @file csl_pscGetHwStatus.c
 *
 * @brief File for functional layer of CSL API CSL_pscGetHwStatus()
 *
 * @path $(CSLPATH)\src\psc
 *
 * @desc The CSL_pscGetHwStatus() function definition & it's associated
 *       functions
 * ============================================================================
 */

/* ============================================================================
 * Revision History
 * ===============
 * 29-Jun-2006 TRR File Created.
 * ============================================================================
 */

#include <csl_psc.h>
#include <csl_pscAux.h>

/** ===========================================================================
 *   @n@b CSL_pscGetHwStatus
 *
 *   @b Description
 *   @n This function is used to get the value of various parameters of the
 *      PSC instance. The value returned depends on the query passed.
 *
 *   @b Arguments
 *   @verbatim
            hPsc            Handle to the PSC instance

            query           Query to be performed

            response        Pointer to buffer to return the data requested by
                            the query passed
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK            - Successful completion of the
 *                                               query
 *
 *   @li                    CSL_ESYS_BADHANDLE - Invalid handle
 *
 *   @li                    CSL_ESYS_INVQUERY  - Query command not supported
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_pscInit() and CSL_pscOpen() must be called successfully in order before calling this function.
 *
 *   <b> Post Condition </b>
 *       Data requested by the query is returned through the variable "response"
 *
 *   @b Modifies
 *   @n  The input argument "response" is modified
 *
 *   @b Example
 *   @verbatim
        CSL_Status     status;
        CSL_PscHandle  hPsc;
        CSL_PscObj     pscObj;
        Uint32         count;

        CSL_pscInit(NULL);
        hPsc = CSL_pscOpen(&pscObj, CSL_PSC, NULL, &status); 

        ...

        status = CSL_pscGetHwStatus(hPsc, CSL_PSC_QUERY_MODULE_STAT,
                                    &count);
        ... 
     @endverbatim
 * ============================================================================
 */
#pragma CODE_SECTION (CSL_pscGetHwStatus, ".text:csl_section:psc");
CSL_Status CSL_pscGetHwStatus (
    CSL_PscHandle        hPsc,
    CSL_PscHwStatusQuery query,
    void                 *response
)
{
    CSL_Status status = CSL_SOK;

    if (hPsc == NULL){
        return CSL_ESYS_BADHANDLE;
    }

    if (response == NULL){
        return CSL_ESYS_INVPARAMS;
    }
    
    switch (query) {
        case CSL_PSC_QUERY_MODULE_STAT:
            CSL_pscGetModuleStat(hPsc, (CSL_PscmoduleState *)response);
            break;

        case CSL_PSC_QUERY_PWRDMN_STAT:
            CSL_pscGetPwrDomainStat(hPsc, (CSL_PscPwrdmnState *)response);
            break;

        case CSL_PSC_QUERY_PWRDMN_TRANS_STAT:
            CSL_pscGetPwrDmnTransStat(hPsc, (CSL_PscPwrDmnTransState *)response);
            break;

        default:
            status = CSL_ESYS_INVQUERY;
		    break;
    }

    return status;
}

