/* ============================================================================
 * Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006
 *
 * Use of this software is controlled by the terms and conditions found in the
 * license agreement under which this software has been supplied.
 * =============================================================================
 */

/** ===========================================================================
 * @file csl_pscHwControl.c
 *
 * @brief File for functional layer of CSL API CSL_pscHwControl()
 *
 * @path $(CSLPATH)\src\psc
 *
 * @desc The CSL_pscHwControl() function definition & it's associated
 *       functions
 *
 */

/* ============================================================================
 * Revision History
 * ===============
 * 29-Jun-2006 TRR File Created.
 * ============================================================================
 */

#include <csl_psc.h>
#include <csl_pscAux.h>

/** ============================================================================
 *   @n@b CSL_pscHwControl
 *
 *   @b Description
 *   @n This function performs various control operations on the PSC instance,
 *      based on the command passed.
 *
 *   @b Arguments
 *   @verbatim
            hPsc         Handle to the PSC instance

            cmd          Operation to be performed on the PSC

            arg          Optional argument as per the control command

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK            - Command execution successful.
 *   @li                    CSL_ESYS_BADHANDLE - Invalid handle
 *   @li                    CSL_ESYS_INVCMD    - Invalid command
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_pscInit() and CSL_pscOpen() must be called successfully in order before calling this function.
 *
 *   <b> Post Condition </b>
 *   @n  Registers of the PSC instance are configured according to the command
 *       and the command arguments. The command determines which registers are
 *       modified.
 *
 *   @b Modifies
 *   @n Registers determined by the command
 *
 *   @b Example
 *   @verbatim
 
        CSL_Status     status;
        CSL_PscHandle  hPsc;
        CSL_PscObj     pscObj;

        CSL_pscInit(NULL);
        hPsc = CSL_pscOpen(&pscObj, CSL_PSC, NULL, &status); 
 
        ...
        status = CSL_pscHwControl(hPsc,CSL_PSC_CMD_ENABLE_MODULE,NULL);
        ...

     @endverbatim
 * ===========================================================================
 */
#pragma CODE_SECTION (CSL_pscHwControl, ".text:csl_section:psc");
CSL_Status  CSL_pscHwControl (
    CSL_PscHandle       hPsc,
    CSL_PscHwControlCmd cmd,
    void                *arg
)
{
    CSL_Status status = CSL_SOK ;

    if (hPsc == NULL) {
        return CSL_ESYS_BADHANDLE;
    }

	if (arg == NULL) {
        return CSL_ESYS_INVPARAMS;
    }

    switch (cmd) {
        /* Enable clock */
        case CSL_PSC_CMD_ENABLE_MODULE:
            status = CSL_PscEnableClock(hPsc, (CSL_PscPeripherals *)arg);
            break;

        /* Enable power domain tansition */
        case CSL_PSC_CMD_PWRDMN_TRNS:
            CSL_PscPwrDomainTrans(hPsc, (CSL_PscPowerDomain *)arg);
            break;

        /* Enable Power domain */
        case CSL_PSC_CMD_ENABLE_PWRDMN:
            status = CSL_PscEnablePwrdmn(hPsc, (CSL_PscPowerDomain *)arg);
            break;

        default:
            status = CSL_ESYS_INVCMD;
	        break;
    }

    return status;
}

