/* =============================================================================
 * Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006
 *
 * Use of this software is controlled by the terms and conditions found in the
 * license agreement under which this software has been supplied.
 *  ===========================================================================
 */

/* ============================================================================
 * @file csl_pscOpen.c
 *
 * @brief File for functional layer of CSL API CSL_pscOpen()
 *
 * @Path $(CSLPATH)\src\psc
 *
 * @desc The CSL_pscOpen() function definition & it's associated
 *       functions
 * ============================================================================
 */

/* ============================================================================
 * Revision History
 * ===============
 * 29-Jun-2006 TRR File Created.
 * ============================================================================
 */

#include <csl_psc.h>

/** ===========================================================================
 * @b CSL_pscOpen
 * @b Description
 * @n This function populates the peripheral data object for the PSC
 *    instance and returns a handle to the instance.
 *    The open call sets up the data structures for the particular instance
 *    of PSC device. The device can be re-opened anytime after it has been
 *    normally closed if so required. The handle returned by this call is
 *    input as an essential argument for rest of the APIs described
 *    for this module.
 *
 * @b Arguments
 * @verbatim
          pscObj          Pointer to PSC object.

          pscNum          Instance of PSC CSL to be opened.
          
          pPscParam       Module specific parameters.
          
          status          Status of the function call
   @endverbatim
 *
 * <b> Return Value </b>  CSL_PscHandle
 * @n                     Valid PSC handle will be returned if
 *                        status value is equal to CSL_SOK.
 *
 * <b> Pre Condition </b>
 * @n  The PSC must be successfully initialized via CSL_pscInit() before calling this function. 
 *
 * <b> Post Condition </b>
 * @n   1. The status is returned in the status variable. If status
 *         returned is
 * @li     CSL_SOK             Valid PSC handle is returned
 * @li     CSL_ESYS_FAIL       The PSC instance is invalid
 *
 *      2. Psc object structure is populated
 *
 * @b Modifies
 * @n   1. The status variable
 *
 *      2. Psc object structure
 *
 * @b Example
 * @verbatim
            CSL_status              status;
            CSL_PscObj              pscObj;
            CSL_PscHandle           hPsc;
            hPsc = CSL_pscOpen(&pscObj, CSL_PSC, NULL, &status);
            ...
   @endverbatim
 * ========================================================================== */
#pragma CODE_SECTION(CSL_pscOpen, ".text:csl_section:psc");
CSL_PscHandle CSL_pscOpen (
    CSL_PscObj   *pPscObj,
    CSL_InstNum  pscNum,
    CSL_PscParam *pPscParam,
    CSL_Status   *pStatus
)
{
    CSL_Status st;
    CSL_PscHandle hPsc;
    CSL_PscBaseAddress baseAddress;

	if (pStatus == NULL) {
        hPsc = (CSL_PscHandle)NULL;
        return hPsc;
    }

    if (pPscObj == NULL) {
        *pStatus = CSL_ESYS_INVPARAMS;
        hPsc    = (CSL_PscHandle)NULL;
        return hPsc;
    }

    st = CSL_pscGetBaseAddress(pscNum, pPscParam, &baseAddress);

    if (st == CSL_SOK) {
        pPscObj->regs = baseAddress.regs;
        pPscObj->perNum = (CSL_InstNum)pscNum;
        hPsc = (CSL_PscHandle)pPscObj;
    }
    else {
        pPscObj->regs = (CSL_PscRegsOvly)NULL;
        pPscObj->perNum = (CSL_InstNum)-1;
        hPsc = (CSL_PscHandle)NULL;
    }

    if (pStatus != NULL) {
        *pStatus = st;
    }

    return hPsc;
}

