 /*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2002, 2003, 2004, 2005, 2006
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
 *   ===========================================================================
 */

/** ===========================================================================
 *  @file csl_semHwControl.c
 *
 *  @brief File for functional layer of CSL API @a CSL_semHwControl()
 *
 *  @path $(CSLPATH)\sem\src
 *
 *  Description
 *    - The @a CSL_semHwControl() function definition & it's associated
 *      functions
 */

/* =============================================================================
 * Revision History
 * ===============
 *  10-Apr-2006 NS updated the file for DOXYGEN compatibiliy
 *
 * =============================================================================
 */

#include <csl_sem.h>

/** ============================================================================
 *   @n@b  CSL_semHwControl
 *
 *   @b Description
 *   @n  The Control Command call writes a command value to the semaphore
 *
 *   @b Arguments
 *   @verbatim
            hSem            Handle to the SEM instance

            ctrlCmd         The command to this API indicates the action to be
                            taken on SEM.

            arg             An optional argument.
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Status info return successful.
 *
 *   <b> Pre Condition </b>
 *   @n Both CSL_semInit() (optional) and CSL_semOpen()must be called
 *   successfully in that order before CSL_semHwControl() can be called.
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n The hardware registers of SEM.
 *
 *   @b Example
 *   @verbatim
      // handle for SEM
      CSL_SemHandle handleSem;
      //  SEM object
      CSL_SemObj            *pSemObj;
      // CSL status
      CSL_Status status;
      CSL_SemParam       pSemParam;
      //Number of SEM resources
      pSemParam.flags = 2


      CSL_semInit(NULL); // Init CSL for SEM module, this step is not required
      ...
      // Open handle for SEM module
      handleSem = CSL_semOpen (pSemObj, CSL_SEM, &pSemParam, &status);
      ...
      status = CSL_semHwControl(handleSem, CSL_SEM_CMD_WRITE_POST_DIRECT, NULL);

     @endverbatim
 * =============================================================================
 */
#pragma CODE_SECTION (CSL_semHwControl, ".text:csl_section:sem");
CSL_Status  CSL_semHwControl(
   CSL_SemHandle            hSem,
   CSL_SemHwControlCmd      ctrlCmd,
   void                     *arg
)
{
	CSL_Status status = CSL_SOK;
	volatile unsigned int *pSemReg;
	int sem;
    CSL_SemOwnerId owner;
    CSL_SemEOISet   eoiSet;
	volatile CSL_SemFlagSetClear_Arg	*pArg;

    if (hSem == NULL) {
        return CSL_ESYS_BADHANDLE;
    }

	sem = hSem->semNum;
	pSemReg = hSem->regs->DIRECT;           //HW_SEM_DIRECT_ACCESS;
	pArg = (CSL_SemFlagSetClear_Arg *)arg;
	eoiSet = *(CSL_SemEOISet *)arg;

	switch(ctrlCmd){
    	case CSL_SEM_CMD_EOI_WRITE:
            pSemReg = &(hSem->regs->EOI); //HW_SEM_EOI_REG_ADDR;
            *pSemReg = CSL_FMK(SEM_EOI_SELECT_INTR_REARM, eoiSet);
			break;

    	case CSL_SEM_CMD_FREE_DIRECT:                  /* Release semaphore            */
			pSemReg[sem] = HW_SEM_RELEASE;
			break;

    	case CSL_SEM_CMD_WRITE_POST_DIRECT:            /* Perform Indirect acquisition */
			pSemReg[sem] = HW_SEM_REQUEST;
			break;

        case CSL_SEM_CMD_FREE_INDIRECT:                  /* Release semaphore            */
            pSemReg = hSem->regs->INDIRECT;
            pSemReg[sem] = HW_SEM_RELEASE;
            break;

        case CSL_SEM_CMD_WRITE_POST_INDIRECT:            /* Perform Indirect acquisition */
            pSemReg = hSem->regs->INDIRECT;
            pSemReg[sem] = HW_SEM_REQUEST;
            break;

        case CSL_SEM_CMD_FREE_QUERY:                  /* Release semaphore            */
            pSemReg = hSem->regs->QUERY;
            pSemReg[sem] = HW_SEM_RELEASE;
            break;

        case CSL_SEM_CMD_WRITE_POST_QUERY:            /* Perform Indirect acquisition */
            pSemReg = hSem->regs->QUERY;
            pSemReg[sem] = HW_SEM_REQUEST;
            break;


		case CSL_SEM_CMD_CLEAR_FLAGS:
			pSemReg = hSem->regs->FLAG; //HW_SEM_FLAG_CLEAR_ADDR;
			owner	= pArg->masterId;
			pSemReg[owner] = pArg->mask;
			break;

		case CSL_SEM_CMD_FLAG_SET:
			pSemReg = hSem->regs->FLAG_SET; //HW_SEM_FLAG_SET_ADDR;
			owner	= pArg->masterId;
			pSemReg[owner] = pArg->mask;
			break;

	  	case CSL_SEM_CMD_CLEAR_ERR:                  /* Clear error condition        */
  			pSemReg = &(hSem->regs->ERR_CLEAR); //HW_SEM_ERR_CLEAR_ADDR;
			*pSemReg = 1;
			break;

        default:
            status = CSL_ESYS_INVCMD;
			break;
	}

	return status;
}
